
/*
Instances of OntologyText hold the current state of the edited text

Constructed after the edit pane is updated from a string holding its current
content, and an integer holding the location of the caret within this string.

The caret location holds the character before which the caret line appears
on the screen. Thus if the string is "John" and the caret location is 2, the
caret line will appear between characters 1 and 2, i.e. between 'o' and 'h'
since strings are numbered from zero.

The text string is split into sentence strings using the '\n' character. This
will mean that empty new lines count as sentences with a string content of "".
On Windows, where new lines are represented by two characters, '\r' followed
by '\n', this means that sentence strings end in '\r' (except for the sentence
on the final line). This character is kept in the sentence string so that we
can calculate in which word of which sentence the caret is located.

The current sentence is calculated by looking at the end positions of each
sentence within the text string. Each sentence instance can compute its own
end position since it is provided with the full sentence string (including
a final '\r' character on Windows) and its start position within the text
(see makeSentences method), taking account of newline characters ('\n').

The message returned by getMessage for the whole text is the same as the
message for the current sentence, obtained through a getMessage method on
Sentence instances.

*/

import java.util.Vector;
import java.util.Collections;
import java.lang.String;


public class OntologyText
{ private String textString;
  private int caretPosition;
  private String[] sentenceStrings;
  private int numberOfSentences;
  private Sentence[] sentences;
  private int currentSentence;
  private Word currentWord;
  private Vector<Verb> verbs;
  private String ontologyIRI;

  public OntologyText(String arg1, int arg2)
  { textString = arg1;
    caretPosition = arg2;
    sentenceStrings = splitText(textString);
    numberOfSentences = sentenceStrings.length;
    findMetadata();
    sentences = makeSentences(sentenceStrings);
    currentSentence = findCurrentSentence();
    currentWord = sentences[currentSentence].getCurrentWord(caretPosition);
  }

  public void setTextString(String stringArg) {textString = stringArg; }
  public void setCaretPosition(int intArg) {caretPosition = intArg; }

  public String getTextString() {return textString; }
  public int getNumberOfSentences() {return numberOfSentences; }
  public String getMessage() {return sentences[currentSentence].getMessage(); }
  public String getContinuations() {return sentences[currentSentence].getContinuations(); }
  public Word getCurrentWord() {return currentWord; }
  public Sentence getCurrentSentence() {return sentences[currentSentence]; }
  public Sentence getSentence(int s) {return sentences[s]; }
  public Vector<Verb> getVerbs() {return verbs; }
  public String getOntologyIRI() {return ontologyIRI; }

  public String getGeneratedTextString()
  { String s = "";
    for (int i=0; i<numberOfSentences; i++)
    { s = s + sentences[i].getGeneratedSentence();
      if (i < numberOfSentences-1) s = s + "\n";
    }
    return s;
  }

  public void display()
  { System.out.println("Text: " + textString);
    System.out.println("Caret: " + caretPosition);
    System.out.println("Number of sentences: " + numberOfSentences);
    System.out.println("Current sentence: " + currentSentence);
    for (int i=0; i<numberOfSentences; i++) sentences[i].display();
  }

  private Sentence[] makeSentences(String[] strings)
  { int pos = 0;
    Sentence sentences[] = new Sentence[strings.length];
    for (int i=0; i<strings.length; i++)
    {  String s = strings[i];
       int len = s.length();
//       if ((s.length()>0) && (s.charAt(s.length()-1)=='\r'))
//       System.out.println("Silly EOL character for string " + i);
       if ((i < strings.length-1) && (len > 0) && (s.charAt(len-1)=='\r'))
       s = s.substring(0, len-1);
       sentences[i] = new Sentence(i, s, pos, verbs);
       pos = pos + s.length() + 1; // Add one for new line character
    }
    return sentences;
  }

  private String[] splitText(String ts)
  { if (ts.length() == 0) ts = "$$$";
    int tsLength = ts.length();
    if (ts.charAt(tsLength-1) == '\n') ts = ts + "$$$";
//    String ss[] = ts.split("\n");
    String ss[] = ts.split("\\r?\\n");
    int ssFinal = ss.length-1;
    if (ss[ssFinal].equals("$$$")) ss[ssFinal] = "";
    return ss;
  }

  private int findCurrentSentence()
  { int sentenceNumber = 0;
    while (sentences[sentenceNumber].getEndPosition()<caretPosition) sentenceNumber++;
    return sentenceNumber;
  }

// Picks up all the verb declarations before interpreting the axiom sentences
// Also picks up OntologyIRI
  private void findMetadata()
  { verbs = new Vector<Verb>();
    ontologyIRI = "Anonymous";
    for (int i=0; i<numberOfSentences; i++)
    if (sentenceStrings[i].startsWith("#"))
    { Sentence sen = new Sentence(i,sentenceStrings[i],0,verbs);
      Word[] words = sen.getWords();
      if (words.length == 3)
      { String feature = words[1].getWordString().toLowerCase();
        if (feature.equals("ontologyiri"))
        ontologyIRI = words[2].getWordString();
        else if (feature.equals("verb"))
        { Tokeniser tokens = new Tokeniser(words[2].getWordString());
          if (tokens.getNumberOfTokens() > 1)
          { Verb verb = new Verb(tokens.getTokenString(0),tokens.getTokenString(1));
            verbs.addElement(verb);
          }
        }
      }
    }
  } 

}
