import java.io.*;

import org.w3c.dom.*;

import javax.xml.parsers.*;

import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;

import java.util.Vector;
import java.util.Collections;
import java.lang.String;

public class OwlXml
{ private Document doc;
  private String xmlString;

  public static final String INTEGER_DATATYPE = "http://www.w3.org/2001/XMLSchema#integer";
  public static final String STRING_DATATYPE = "http://www.w3.org/2001/XMLSchema#string";

  public OwlXml(OntologyText ontext, Lexicon lex)
  { try 
    { // Obtain a new document
      DocumentBuilderFactory dbfac = DocumentBuilderFactory.newInstance();
      DocumentBuilder docBuilder = dbfac.newDocumentBuilder();
      doc = docBuilder.newDocument();

      // Root of the XML tree
      Element root = doc.createElement("Ontology");
      doc.appendChild(root);
      root.setAttribute("ontologyIRI", "http://www.semanticweb.org/ont23.owl");

      // Example of how to add a comment
      Comment comment = doc.createComment("Created by SWAT Natural Language Editor");
      root.appendChild(comment);

      // Transcode the lexicon
      Vector<String> individuals = lex.getIndividualList();
      Vector<String> classes = lex.getClassList(OwlEnglish.SINGULAR);
      Vector<String> objectProperties = lex.getObjectPropertyList(OwlEnglish.SINGULAR);
      Vector<String> dataProperties = lex.getDataPropertyList(OwlEnglish.SINGULAR);

      // Declarations
      for (int i=0; i<individuals.size(); i++)
      makeDeclaration(root, "NamedIndividual", individuals.elementAt(i));

      for (int i=0; i<classes.size(); i++)
      makeDeclaration(root, "Class", classes.elementAt(i));

      for (int i=0; i<objectProperties.size(); i++)
      makeDeclaration(root, "ObjectProperty", objectProperties.elementAt(i));

      for (int i=0; i<dataProperties.size(); i++)
      makeDeclaration(root, "DataProperty", dataProperties.elementAt(i));

      // Annotations
      for (int i=0; i<individuals.size(); i++)
      makeAnnotation(root, "namedIndividual", individuals.elementAt(i));

      for (int i=0; i<classes.size(); i++)
      makeAnnotation(root, "class", classes.elementAt(i));

      for (int i=0; i<objectProperties.size(); i++)
      makeAnnotation(root, "objectProperty", objectProperties.elementAt(i));

      for (int i=0; i<dataProperties.size(); i++)
      makeAnnotation(root, "dataProperty", dataProperties.elementAt(i));

     // Transcode the OwlTerms for complete axioms only
      Sentence sent;
      OwlTerm term;
      for (int i=0; i<ontext.getNumberOfSentences(); i++)
      { sent = ontext.getSentence(i);
        term = sent.getOwlTerm();
        if ((sent.sentenceIsCorrect() == true) &&
            (term.getFunctor() != OwlTerm.COMMENT) &&
            (term.getFunctor() != OwlTerm.METADATA))
        transcodeTerm(term, doc, root);
      }            

      // Set up a transformer to realise XML tree as a string
      TransformerFactory transfac = TransformerFactory.newInstance();
      Transformer trans = transfac.newTransformer();
      trans.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
      trans.setOutputProperty(OutputKeys.INDENT, "yes");

      // Apply the transformer to generate a string
      StringWriter sw = new StringWriter();
      StreamResult result = new StreamResult(sw);
      DOMSource source = new DOMSource(doc);
      trans.transform(source, result);
      xmlString = sw.toString();
    }
    catch (Exception e) { System.out.println(e); }
  }

  public String getXmlString() {return xmlString; }

  private void transcodeTerm(OwlTerm term, Document doc, Element root)
    { int functor = term.getFunctor();
      String s = OwlTerm.functorString[functor];
      Element element = doc.createElement(s);
      root.appendChild(element);
      int nargs = term.getNumberOfArguments();
      if (nargs == 0) 
      { String name = term.getEntityName();
        if (functor == OwlTerm.LITERAL)
        { String literal = makeLiteral(name);
          Text text = doc.createTextNode(literal);
          element.appendChild(text);
          if ((name.length() > 0) && (name.charAt(0) == '"'))
          element.setAttribute("datatypeIRI",STRING_DATATYPE);
          else element.setAttribute("datatypeIRI",INTEGER_DATATYPE);
        }
        else 
        { String iri = makeIRI(name);
          element.setAttribute("IRI", iri);
        }
      }
      else if (functor == OwlTerm.CLASS_ASSERTION)
      { transcodeTerm(term.getArgument(1), doc, element);
        transcodeTerm(term.getArgument(0), doc, element);
      }
      else if ((functor == OwlTerm.OBJECT_MAX_CARDINALITY) ||
               (functor == OwlTerm.OBJECT_MIN_CARDINALITY) ||
               (functor == OwlTerm.OBJECT_EXACT_CARDINALITY))
      { element.setAttribute("cardinality",digitise(term.getEntityName()));
        transcodeTerm(term.getArgument(0), doc, element);
        transcodeTerm(term.getArgument(1), doc, element);
      }
      else for (int i=0; i<nargs; i++)
      transcodeTerm(term.getArgument(i), doc, element);
    }

  private void makeDeclaration(Element root, String label, String name)
    { Element element = doc.createElement("Declaration");
      root.appendChild(element);
      Element arg = doc.createElement(label);
      element.appendChild(arg);
      String iri = makeIRI(name);
      arg.setAttribute("IRI", iri);
    }

  private void makeAnnotation(Element root, String label, String name)
    { Element element = doc.createElement("AnnotationAssertion");
      root.appendChild(element);

      Element arg1 = doc.createElement("AnnotationProperty");
      String uri = "http://www.swatproject.org/" + label;
      arg1.setAttribute("IRI", uri);
      element.appendChild(arg1);

      Element arg2 = doc.createElement("IRI");
      String iri = makeIRI(name);
      arg2.setTextContent(iri);
      element.appendChild(arg2);

      Element arg3 = doc.createElement("Literal");
      arg3.setTextContent(name);
      element.appendChild(arg3);
    }

  private String makeIRI(String name)
  { String iri = "#" + name.replace(' ','_');
    if (iri.equals("#thing")) iri = "http://www.w3.org/2002/07/owl#Thing";
    return iri;
  }

  private String makeLiteral(String name)
  { String literal = name;
    if ((name.length()>1) && (name.charAt(0) == '"'))
    literal = name.substring(1,name.length()-1);
    return literal;
  }

  private String digitise(String integer)
  { String s = integer;
    for (int i=0; i<Word.integerWord.length; i++)
    if (Word.integerWord[i].equals(integer)) s = "" + i;
    return s;
  }

  public void saveXML(String filename)
  { FileOutputStream out;
    PrintStream p;
    try
    { out = new FileOutputStream(filename);
      p = new PrintStream( out );
      p.println (xmlString);
      p.close();
    }
    catch (Exception e)
    {System.err.println ("Error writing to file");
    }
  }

  public void display()
  { System.out.println("XML output:\n" + xmlString);
  }


}