
/*
Instances of Sentence hold information about sentences in the edited text

When sentences are constructed they are given their number in the text,
(counting from zero), their string content (possibly including '\r' on
Windows, but excluding '\n'), and their starting position within the
whole text string.

Sentence objects are responsible for calculating their own end positions
within the whole text string, tokenising the sentence string into words,
and analysing the sentence. The analysis consists of features attached to
the word objects: each word has a category, such as noun or verb, and a
state, which is the node in the finite-state transducer which is reached
after consuming the word.

Sentence objects return messages based on the state of the last completed
word. These are obtained by calling a getMessage method on that word object.

Sentences are split into words by space separators. This requires some
preliminary work on the sentence string, which may contain the following
problems:
-- Line feed character '\r' at end under Windows
-- Leading or trailing spaces
-- Extra spaces between consecutive words
-- Punctuation (especially full stop) directly following words
Thus if we get the input " Rover   is a dog.\r" we need to revise the
string to "Rover is a dog ." before a split based on space will work.

*/

import java.util.Vector;
import java.util.Collections;

public class Sentence
{ private String textString;
  private int startPosition, endPosition, sentenceNumber;
  private int numberOfCharacters, numberOfWords;
  private String[] wordStrings;
  private Word[] word;
  private OwlTerm axiom;     // OWL term for the sentence
  private boolean correctSentence;
  private Vector<Verb> verbs;
  private int listType;      // UNSPECIFIED|INTERSECTION|UNION

// Types of list (a sentence cannot contain both intersection and union)
  public static final int UNSPECIFIED = 0;
  public static final int INTERSECTION = 1;
  public static final int UNION = 2;

// States of the transducer
  public static final int REJECT = 0;
  public static final int START = 1;
  public static final int ISUB = 2;
  public static final int CSUB = 3;
  public static final int SUBJECT = 4;
  public static final int COPULAR = 5;
  public static final int AUX = 6;
  public static final int NPLIST = 7;
  public static final int COMPLEMENT = 8;
  public static final int NPLISTLOOP = 9;
  public static final int VPLIST = 10;
  public static final int ILIST = 11;
  public static final int ILISTOBJ = 12;
  public static final int LLISTOBJ = 13;
  public static final int CLIST = 14;
  public static final int CLISTOBJ = 15;
  public static final int VPLISTLOOP = 16;
  public static final int AUXLIST = 17;
  public static final int VPCHAINLOOP = 18;
  public static final int AUXCHAIN = 19;
  public static final int VPCHAIN = 20;
  public static final int ICHAIN = 21;
  public static final int ICHAINOBJ = 22;
  public static final int LCHAINOBJ = 23;
  public static final int CCHAIN = 24;
  public static final int CCHAINOBJ = 25;
  public static final int FINISH = 26;
// Additional states for comments and metadata
  public static final int COMMENT = 27;
  public static final int METADATA = 28;
  public static final int METAVALUE = 29;
// Additional states for disjoint and equivalent classes
  public static final int DSUB = 30;
  public static final int DSUBJECT = 31;
  public static final int DCOPULAR = 32;
  public static final int DCOMP = 33;
  public static final int DCOMPLEMENT = 34;
  public static final int DEFINED = 35;
  public static final int DEFAS = 36;
// Additional states for domain and range statements
  public static final int PROPSUBJ = 37;
  public static final int PROPTHAT = 38;
  public static final int DOMAINAUX = 39;
  public static final int DOMAIN = 40;
  public static final int DATADOMAIN = 41;
  public static final int DOMAINVP = 42;
  public static final int RANGE = 43;
  public static final int RANGEAUX = 44;
  public static final int RANGEVP = 45;
  public static final int PROPOBJ = 46;
  public static final int CPROP = 47;
  public static final int CPROPOBJ = 48;
// Additional states for universal and numerical restrictions
  public static final int ATLIST = 49;
  public static final int EQUALITYLIST = 50;
  public static final int INEQUALITYLIST = 51;
  public static final int ATCHAIN = 52;
  public static final int EQUALITYCHAIN = 53;
  public static final int INEQUALITYCHAIN = 54;
// Additional states for statements about object properties
  public static final int PROPINTRO = 55;
  public static final int OPENPROP = 56;
  public static final int AUXPROP = 57;
  public static final int VPPROP = 58;
  public static final int CLOSEPROP = 59;
  public static final int PROPCOP = 60;
  public static final int PROPATTR = 61;
// Additional states for statements about data properties
  public static final int DATAPROPINTRO = 62;
  public static final int OPENDATAPROP = 63;
  public static final int AUXDATAPROP = 64;
  public static final int VPDATAPROP = 65;
  public static final int CLOSEDATAPROP = 66;
  public static final int DATAPROPCOP = 67;
  public static final int DATAPROPATTR = 68;
// Including object property relations
  public static final int PROPINTROREL = 69;
  public static final int OPENPROPREL = 70;
  public static final int AUXPROPREL = 71;
  public static final int VPPROPREL = 72;
  public static final int CLOSEPROPREL = 73;
  public static final int PROPCOPREL = 74;
  public static final int PROPATTRREL = 75;
// Special case of sub-property
  public static final int PROPSUBA = 76;
  public static final int PROPSUBB = 77;
  public static final int PROPINTROSUB = 78;
  public static final int OPENPROPSUB = 79;
  public static final int AUXPROPSUB = 80;
  public static final int VPPROPSUB = 81;
  public static final int CLOSEPROPSUB = 82;
// Including data property relations
  public static final int DATAPROPINTROREL = 83;
  public static final int OPENDATAPROPREL = 84;
  public static final int AUXDATAPROPREL = 85;
  public static final int VPDATAPROPREL = 86;
  public static final int CLOSEDATAPROPREL = 87;
  public static final int DATAPROPCOPREL = 88;
  public static final int DATAPROPATTRREL = 89;
// Special case of sub-data-property
  public static final int DATAPROPSUBA = 90;
  public static final int DATAPROPSUBB = 91;
  public static final int DATAPROPINTROSUB = 92;
  public static final int OPENDATAPROPSUB = 93;
  public static final int AUXDATAPROPSUB = 94;
  public static final int VPDATAPROPSUB = 95;
  public static final int CLOSEDATAPROPSUB = 96;

// Roles for syntax highlighting
  public static final int ERROR = 0;
  public static final int SUGAR = 1;
  public static final int INDIVIDUAL = 2;
  public static final int CLASS = 3;
  public static final int PROPERTY = 4;
  public static final int LITERAL = 5;
  public static final int RETAIN = 6;
  public static final int REVISE = 7;
  public static final int SUGARBACK = 8;
  public static final int REVISEBACK = 9;
  public static final int SUGARBACKTWO = 10;

// Each transition is assigned a role for colour-coding 
  private static int role[][] =
   {{ START, ISUB, INDIVIDUAL },
    { START, CSUB, SUGAR },
    { START, SUBJECT, INDIVIDUAL },
    { ISUB, SUBJECT, INDIVIDUAL },
    { CSUB, SUBJECT, CLASS },
    { SUBJECT, SUBJECT, RETAIN },
    { SUBJECT, COPULAR, SUGAR },
    { SUBJECT, AUX, PROPERTY },
    { SUBJECT, VPLIST, PROPERTY },
    { COPULAR, NPLIST, SUGAR },
    { COPULAR, VPLIST, REVISE },
    { NPLIST, COMPLEMENT, CLASS },
    { COMPLEMENT, COMPLEMENT, CLASS },
    { COMPLEMENT, NPLISTLOOP, SUGAR },
    { NPLISTLOOP, NPLIST, SUGAR },
    { COMPLEMENT, VPLISTLOOP, SUGAR },
    { COMPLEMENT, FINISH, SUGAR },
    { AUX, VPLIST, PROPERTY },
    { VPLIST, VPLIST, PROPERTY },
    { VPLIST, ILIST, INDIVIDUAL },
    { VPLIST, CLIST, SUGAR },
    { VPLIST, ILISTOBJ, INDIVIDUAL },
    { ILIST, ILISTOBJ, INDIVIDUAL },
    { ILISTOBJ, ILISTOBJ, INDIVIDUAL },
    { CLIST, CLISTOBJ, CLASS },
    { CLISTOBJ, CLISTOBJ, CLASS },
    { ILISTOBJ, FINISH, SUGAR },
    { CLISTOBJ, FINISH, PROPERTY },
    { ILISTOBJ, VPLISTLOOP, SUGAR },
    { CLISTOBJ, VPLISTLOOP, SUGAR },
    { VPLISTLOOP, AUXLIST, PROPERTY },
    { AUXLIST, VPLIST, PROPERTY },
    { VPLISTLOOP, VPLIST, PROPERTY },
    { VPLIST, VPLIST, PROPERTY },
    { CLISTOBJ, VPCHAINLOOP, SUGAR },
    { VPCHAINLOOP, AUXCHAIN, PROPERTY },
    { VPCHAINLOOP, VPCHAIN, PROPERTY },
    { AUXCHAIN, VPCHAIN, PROPERTY },
    { VPCHAIN, ICHAIN, INDIVIDUAL },
    { VPCHAIN, CCHAIN, SUGAR },
    { VPCHAIN, ICHAINOBJ, INDIVIDUAL },
    { VPCHAIN, VPCHAIN, PROPERTY },
    { ICHAIN, ICHAINOBJ, INDIVIDUAL },
    { ICHAINOBJ, ICHAINOBJ, INDIVIDUAL },
    { CCHAIN, CCHAINOBJ, CLASS },
    { CCHAINOBJ, CCHAINOBJ, CLASS },
    { ICHAINOBJ, FINISH, SUGAR },
    { CCHAINOBJ, FINISH, SUGAR },
    { CCHAINOBJ, VPCHAINLOOP, SUGAR },
    { VPLIST, LLISTOBJ, LITERAL },
    { VPCHAIN, LCHAINOBJ, LITERAL },
    { LLISTOBJ, VPLISTLOOP, SUGAR },
    { LLISTOBJ, FINISH, SUGAR },
    { LCHAINOBJ, FINISH, SUGAR },
    { START, COMMENT, SUGAR },
    { START, METADATA, SUGAR },
    { COMMENT, COMMENT, SUGAR },
    { COMMENT, FINISH, SUGAR },
    { METADATA, METADATA, SUGAR },
    { METADATA, FINISH, SUGAR },
    { START, DSUB, SUGAR },
    { DSUB, DSUBJECT, CLASS },
    { DSUBJECT, DSUBJECT, CLASS },
    { DSUBJECT, DCOPULAR, SUGAR },
    { DCOPULAR, DCOMP, SUGAR },
    { DCOMP, DCOMPLEMENT, CLASS },
    { DCOMPLEMENT, DCOMPLEMENT, CLASS },
    { DCOMPLEMENT, FINISH, SUGAR },
    { COPULAR, DEFINED, SUGAR },
    { DEFINED, DEFAS, SUGAR },
    { DEFAS, NPLIST, SUGAR },
    { START, PROPSUBJ, SUGAR },
    { PROPSUBJ, PROPTHAT, SUGAR },
    { PROPTHAT, DOMAINAUX, PROPERTY },
    { PROPTHAT, DOMAIN, PROPERTY },
    { PROPTHAT, RANGE, SUGAR },
    { DOMAINAUX, DOMAIN, PROPERTY },
    { DOMAIN, DOMAIN, PROPERTY },
    { DOMAIN, DOMAINVP, SUGAR },
    { RANGE, RANGEAUX, PROPERTY },
    { RANGE, RANGEVP, PROPERTY },
    { RANGEAUX, RANGEVP, PROPERTY },
    { RANGEVP, RANGEVP, PROPERTY },
    { RANGEVP, PROPOBJ, SUGAR },
    { DOMAINVP, PROPOBJ, SUGAR },
    { PROPOBJ, CPROP, SUGAR },
    { CPROP, CPROPOBJ, CLASS },
    { CPROPOBJ, CPROPOBJ, CLASS },
    { CPROPOBJ, FINISH, SUGAR },
    { VPLIST, ATLIST, PROPERTY },
    { VPLIST, EQUALITYLIST, SUGAR },
    { ATLIST, VPLIST, PROPERTY },
    { ATLIST, ATLIST, PROPERTY },
    { ATLIST, CLIST, SUGAR },
    { ATLIST, EQUALITYLIST, SUGARBACK },
    { ATLIST, INEQUALITYLIST, SUGARBACK },
    { EQUALITYLIST, CLIST, SUGAR },
    { INEQUALITYLIST, CLIST, SUGAR },
    { VPCHAIN, ATCHAIN, PROPERTY },
    { VPCHAIN, EQUALITYCHAIN, SUGAR },
    { ATCHAIN, VPCHAIN, PROPERTY },
    { ATCHAIN, ATCHAIN, PROPERTY },
    { ATCHAIN, CCHAIN, SUGAR },
    { ATCHAIN, EQUALITYCHAIN, SUGARBACK },
    { ATCHAIN, INEQUALITYCHAIN, SUGARBACK },
    { EQUALITYCHAIN, CCHAIN, SUGAR },
    { INEQUALITYCHAIN, CCHAIN, SUGAR },
    { DEFINED, VPLIST, REVISEBACK },
    { DOMAIN, DATADOMAIN, SUGAR },
    { DATADOMAIN, DOMAINVP, SUGAR },
    { ISUB, PROPINTRO, INDIVIDUAL },
    { PROPINTRO, SUBJECT, INDIVIDUAL },
    { PROPINTRO, OPENPROP, SUGARBACKTWO },
    { OPENPROP, AUXPROP, PROPERTY },
    { OPENPROP, VPPROP, PROPERTY },
    { AUXPROP, VPPROP, PROPERTY },
    { VPPROP, VPPROP, PROPERTY },
    { VPPROP, CLOSEPROP, SUGAR },
    { CLOSEPROP, PROPCOP, SUGAR },
    { PROPCOP, PROPATTR, SUGAR },
    { PROPATTR, FINISH, SUGAR },
    { ISUB, DATAPROPINTRO, INDIVIDUAL },
    { DATAPROPINTRO, SUBJECT, INDIVIDUAL },
    { DATAPROPINTRO, OPENDATAPROP, SUGARBACKTWO },
    { OPENDATAPROP, AUXDATAPROP, PROPERTY },
    { OPENDATAPROP, VPDATAPROP, PROPERTY },
    { AUXDATAPROP, VPDATAPROP, PROPERTY },
    { VPDATAPROP, VPDATAPROP, PROPERTY },
    { VPDATAPROP, CLOSEDATAPROP, SUGAR },
    { CLOSEDATAPROP, DATAPROPCOP, SUGAR },
    { DATAPROPCOP, DATAPROPATTR, SUGAR },
    { DATAPROPATTR, FINISH, SUGAR },
    { CLOSEPROP, PROPINTROREL, SUGAR },
    { PROPINTROREL, OPENPROPREL, SUGAR },
    { OPENPROPREL, AUXPROPREL, PROPERTY },
    { OPENPROPREL, VPPROPREL, PROPERTY },
    { AUXPROPREL, VPPROPREL, PROPERTY },
    { VPPROPREL, VPPROPREL, PROPERTY },
    { VPPROPREL, CLOSEPROPREL, SUGAR },
    { CLOSEPROPREL, PROPCOPREL, SUGAR },
    { PROPCOPREL, PROPATTRREL, SUGAR },
    { PROPATTRREL, FINISH, SUGAR },
    { PROPCOP, PROPSUBA, SUGAR },
    { PROPSUBA, PROPSUBB, SUGAR },
    { PROPSUBB, PROPINTROSUB, SUGAR },
    { PROPINTROSUB, OPENPROPSUB, SUGAR },
    { OPENPROPSUB, AUXPROPSUB, PROPERTY },
    { OPENPROPSUB, VPPROPSUB, PROPERTY },
    { AUXPROPSUB, VPPROPSUB, PROPERTY },
    { VPPROPSUB, VPPROPSUB, PROPERTY },
    { VPPROPSUB, CLOSEPROPSUB, SUGAR },
    { CLOSEPROPSUB, FINISH, SUGAR },
    { CLOSEDATAPROP, DATAPROPINTROREL, SUGAR },
    { DATAPROPINTROREL, OPENDATAPROPREL, SUGAR },
    { OPENDATAPROPREL, AUXDATAPROPREL, PROPERTY },
    { OPENDATAPROPREL, VPDATAPROPREL, PROPERTY },
    { AUXDATAPROPREL, VPDATAPROPREL, PROPERTY },
    { VPDATAPROPREL, VPDATAPROPREL, PROPERTY },
    { VPDATAPROPREL, CLOSEDATAPROPREL, SUGAR },
    { CLOSEDATAPROPREL, DATAPROPCOPREL, SUGAR },
    { DATAPROPCOPREL, DATAPROPATTRREL, SUGAR },
    { DATAPROPATTRREL, FINISH, SUGAR },
    { DATAPROPCOP, DATAPROPSUBA, SUGAR },
    { DATAPROPSUBA, DATAPROPSUBB, SUGAR },
    { DATAPROPSUBB, DATAPROPINTROSUB, SUGAR },
    { DATAPROPINTROSUB, OPENDATAPROPSUB, SUGAR },
    { OPENDATAPROPSUB, AUXDATAPROPSUB, PROPERTY },
    { OPENDATAPROPSUB, VPDATAPROPSUB, PROPERTY },
    { AUXDATAPROPSUB, VPDATAPROPSUB, PROPERTY },
    { VPDATAPROPSUB, VPDATAPROPSUB, PROPERTY },
    { VPDATAPROPSUB, CLOSEDATAPROPSUB, SUGAR },
    { CLOSEDATAPROPSUB, FINISH, SUGAR }
  };

// Messages associated with each state transition
  private static String transitionMessage[] =
    { "0 No transition",
      "1 Definite article opening an individual name in the subject",
      "2 Indefinite article preceding a class name in the subject",
      "3 Proper name opening an individual name in the subject",
      "4 Continuing an individual name in the subject after THE",
      "5 Opening a class name in the subject",
      "6 Continuing an individual or class name in the subject",
      "7 Copular, or auxiliary opening a property name after the subject",
      "8 Auxiliary opening a property name after the subject",
      "9 Finite verb opening a property name after the subject",
      "10 Indefinite article preceding the complement",
      "11 Continuing a property name after the subject and auxiliary IS",
      "12 Opening a class name in the complement",
      "13 Continuing a class name in the complement",
      "14 Conjunction preparing to extend a class list",
      "15 Indefinite article preceding a class name in a class list",
      "16 Relative pronoun preceding a restriction in the complement",
      "17 Full stop completing a correct sentence with no restrictions",
      "18 Continuing a property name after an auxiliary",
      "19 Continuing a property name after the subject and auxiliary HAS",
      "20 Definite article opening an individual name in a restriction",
      "21 Indefinite article preceding a class name in a restriction",
      "22 Proper name opening an individual name in a restriction",
      "23 Continuing an individual name in a restriction after THE",
      "24 Continuing an individual name in a restriction",
      "25 Opening a class name in a restriction",
      "26 Continuing a class name in a restriction",
      "27 Full stop completing a correct sentence with restriction(s)",
      "28 Full stop completing a correct sentence with restriction(s)",
      "29 Conjunction preparing to extend a restriction list",
      "30 Conjunction preparing to extend a restriction list",
      "31 Auxiliary opening a property name in a restriction list",
      "32 Continuing a property name in a restriction list after an auxiliary",
      "33 Finite verb opening a property name in a restriction list",
      "34 Continuing a property name in a restriction",
      "35 Relative pronoun preceding a restriction chain",
      "36 Auxiliary opening a property name in a restriction chain",
      "37 Finite verb opening a property name in a restriction chain",
      "38 Continuing a property name in a restriction chain after an auxiliary",
      "39 Definite article opening an individual name in a restriction chain",
      "40 Indefinite article preceding a class name in a restriction chain",
      "41 Proper name opening an individual name in a restriction chain",
      "42 Continuing a property name in a restriction chain",
      "43 Continuing an individual name in a restriction chain after THE",
      "44 Continuing an individual name in a restriction chain",
      "45 Opening a class name in a restriction chain",
      "46 Continuing a class name in a restriction chain",
      "47 Full stop completing a correct sentence with a restriction chain",
      "48 Full stop completing a correct sentence with a restriction chain",
      "49 Relative pronoun preparing to extend a restriction chain",
      "50 Number or string specifying the value of a data property",
      "51 Number or string specifying the value of a data property",
      "52 Conjunction preparing to extend a restriction list",
      "53 Full stop completing a correct sentence with restriction(s)",
      "54 Full stop completing a correct sentence with a restriction chain",
      "55 Percentage character introducing comments message",
      "56 Hash character introducing metadata statement",
      "57 Continuation of comments message",
      "58 Continuation of comments message",
      "59 Continuation of metadata statement",
      "60 Continuation of metadata statement",
      "61 Start of disjoint classes statement",
      "62 Opening subject class name in disjoint classes statement",
      "63 Continuing subject class name in disjoint classes statement",
      "64 Copular in disjoint classes statement",
      "65 Opening complement in disjoint classes statement",
      "66 Opening complement class name in disjoint classes statement",
      "67 Continuing complement class name in disjoint classes statement",
      "68 Completing disjoint classes statement",
      "69 Signalling an equivalent classes statement",
      "70 Continuing an equivalent classes statement",
      "71 Opening complement in equivalent classes statement",
      "72 Opening subject in domain/range statement",
      "73 Continuing subject in domain/range statement",
      "74 Opening property name in domain statement",
      "75 Opening property name in domain statement",
      "76 Continuing subject in range statement",
      "77 Continuing property name in domain statement",
      "78 Continuing property name in domain statement",
      "79 Continuing subject in domain statement",
      "80 Opening property name in range statement",
      "81 Opening property name in range statement",
      "82 Continuing property name in range statement",
      "83 Continuing property name in range statement",
      "84 Opening complement in range statement",
      "85 Opening complement in domain statement",
      "86 Continuing complement in domain/range statement",
      "87 Opening class name in complement of domain/range statement",
      "88 Continuing class name in complement of domain/range statement",
      "89 Completing a domain/range statement",
      "90 Continuing property name or starting an inequality quantifier",
      "91 Starting an equality quantifier",
      "92 Continuing a property name",
      "93 Continuing property name or starting an inequality quantifier",
      "94 Continuing an existential or universal restriction in a restriction list",
      "95 Starting an equality quantifier",
      "96 Continuing an inequality quantifier",
      "97 Completing an equality quantifier",
      "98 Completing an inequality quantifier",
      "99 Continuing property name or starting an inequality quantifier",
      "100 Starting an equality quantifier",
      "101 Continuing a property name",
      "102 Continuing property name or starting an inequality quantifier",
      "103 Continuing an existential or universal restriction in a restriction chain",
      "104 Starting an equality quantifier",
      "105 Continuing an inequality quantifier",
      "106 Completing an equality quantifier",
      "107 Completing an inequality quantifier",
      "108 Continuing a property name including DEFINED",
      "109 Continuing from a data property name in a domain statement",
      "110 Continuing from a data property name in a domain statement",
      "111 Either continuing an individual name or opening a property statement",
      "112 Continuing an individual name",
      "113 Opening a reference to a property",
      "114 Opening a property name with an auxiliary",
      "115 Opening a property name with a verb in the present tense",
      "116 Continuing a property name in a property statement",
      "117 Continuing a property name in a property statement",
      "118 Closing a reference to a property",
      "119 Continuing a property statement",
      "120 Specifying a property attribute such as functional",
      "121 Closing a property statement",
      "122 Either continuing an individual name or opening a data-property statement",
      "123 Continuing an individual name",
      "124 Opening a reference to a data-property",
      "125 Opening a data-property name with an auxiliary",
      "126 Opening a data-property name with a verb in the present tense",
      "127 Continuing a data-property name in a data-property statement",
      "128 Continuing a data-property name in a data-property statement",
      "129 Closing a reference to a data-property",
      "130 Continuing a data-property statement",
      "131 Specifying the data-property attribute functional",
      "132 Closing a data-property statement",
      "133 Conjunction in a property relation",
      "134 Opening a reference to a second property",
      "135 Opening a property name in a property relation with auxiliary",
      "136 Opening a property name in a property relation with verb",
      "137 Continuing a property name in a relation after auxiliary",
      "138 Continuing a property name in a relation after verb",
      "139 Closing a reference to a second property",
      "140 Copular before defining a property relation",
      "141 Specifying a property relation",
      "142 Closing a property relation statement",
      "143 Converting to a sub-property statement",
      "144 Continuing a sub-property statement",
      "145 Continuing a sub-property statement",
      "146 Opening a reference to a second property in a sub-property statement",
      "147 Opening a property name in a sub-property statement with auxiliary",
      "148 Opening a property name in a sub-property statement with verb",
      "149 Continuing a property name in a sub-property statement after auxiliary",
      "150 Continuing a property name in a sub-property statement after verb",
      "151 Closing a reference to a second property in a sub-property statement",
      "152 Closing a sub-property statement",
      "153 Conjunction in a data-property relation",
      "154 Opening a reference to a second data-property",
      "155 Opening a property name in a data-property relation with auxiliary",
      "156 Opening a property name in a data-property relation with verb",
      "157 Continuing a property name in a relation after auxiliary",
      "158 Continuing a property name in a relation after verb",
      "159 Closing a reference to a second data-property",
      "160 Copular before defining a data-property relation",
      "161 Specifying a data-property relation",
      "162 Closing a data-property relation statement",
      "163 Converting to a sub-data-property statement",
      "164 Continuing a sub-vproperty statement",
      "165 Continuing a sub-data-property statement",
      "166 Opening a reference to a second property in a sub-data-property statement",
      "167 Opening a property name in a sub-data-property statement with auxiliary",
      "168 Opening a property name in a sub-data-property statement with verb",
      "169 Continuing a property name in a sub-data-property statement after auxiliary",
      "170 Continuing a property name in a sub-data-property statement after verb",
      "171 Closing a reference to a second property in a sub-data-property statement",
      "172 Closing a sub-data-property statement"
    };

// Associates options for completing/continuing the sentence with each state
  private static String options[][] =
   { { "No options" },                               // 0 REJECT
     { "[Individual] is a [class].",
       "[Individual] [has-property] [Individual].",
       "[Individual] [has-data-property] [literal].",
       "[Individual] [has-property] a [class].",
       "[Individual] [has-property] only [class].",
       "[Individual] [has-property] exactly [integer] [class].",
       "[Individual] [has-property] at least [integer] [class].",
       "[Individual] [has-property] at most [integer] [class].",
       "A [class] is a [class].",
       "A [class] [has-property] [Individual].",
       "A [class] [has-data-property] [literal].",
       "A [class] [has-property] a [class].",
       "A [class] [has-property] only [class].",
       "A [class] [has-property] exactly [integer] [class].",
       "A [class] [has-property] at least [integer] [class].",
       "A [class] [has-property] at most [integer] [class].",
       "No [class] is a [class].",
       "A [class] is defined as a [class].",
       "Anything that [has-property] something is a [class].",
       "Anything that something [has-property] is a [class].",
       "Anything that [has-data-property] some value is a [class].",
       "The property <[has-property]> is [attribute].",
       "The property <[has-property]> is a sub-property of <[has-property]>.",
       "The properties <[has-property]> and <[has-property]> are disjoint.",
       "The properties <[has-property]> and <[has-property]> are equivalent.",
       "The properties <[has-property]> and <[has-property]> are inverse.",
       "The data-property <[has-data-property]> is functional.",
       "The data-property <[has-data-property]> is a sub-property of <[has-data-property]>.",
       "The data-properties <[has-data-property]> and <[has-data-property]> are disjoint.",
       "The data-properties <[has-data-property]> and <[has-data-property]> are equivalent." }, // 1 START
     { "No options" },                               // 2 ISUB
     { "[class] is a [class].",
       "[class] is defined as a [class].",
       "[class] [has-property] [Individual].",
       "[class] [has-data-property] [literal].",
       "[class] [has-property] a [class].",
       "[class] [has-property] only [class].",
       "[class] [has-property] exactly [integer] [class].",
       "[class] [has-property] at least [integer] [class].",
       "[class] [has-property] at most [integer] [class]." }, // 3 CSUB
     { "is a [class].",
       "is defined as a [class].",
       "[has-property] [Individual].",
       "[has-data-property] [literal].",
       "[has-property] a [class].",
       "[has-property] only [class].",
       "[has-property] exactly [integer] [class].",
       "[has-property] at least [integer] [class].",
       "[has-property] at most [integer] [class]." }, // 4 SUBJECT
     { "a [class].",
       "defined as a [class]." },                  // 5 COPULAR
     { "No options" },                               // 6 AUX
     { "[class]." },                                 // 7 NPLIST
     { "and a [class].",
       "or a [class].",
       "that [has-property] [Individual].",
       "that [has-data-property] [literal].",
       "that [has-property] a [class].",           
       "that [has-property] only [class].",
       "that [has-property] exactly [integer] [class].",
       "that [has-property] at least [integer] [class].",
       "that [has-property] at most [integer] [class]." }, // 8 COMPLEMENT
     { "a [class]." },                               // 9 NPLISTLOOP
     { "[Individual].",
       "[literal].",
       "a [class].",
       "only [class].",
       "exactly [integer] [class].",
       "at least [integer] [class].",
       "at most [integer] [class]." },               // 10 VPLIST
     { "No options" },                               // 11 ILIST
     { "and [has-property] [Individual].",
       "and [has-data-property] [literal].",
       "and [has-property] a [class].",
       "or [has-property] [Individual].",
       "or [has-data-property] [literal].",
       "or [has-property] a [class]." },             // 12 ILISTOBJ
     { "and [has-property] [Individual].",
       "and [has-data-property] [literal].",
       "and [has-property] a [class].",
       "or [has-property] [Individual].",
       "or [has-data-property] [literal].",
       "or [has-property] a [class]." },             // 13 LLISTOBJ
     { "[class]." },                                 // 14 CLIST
     { "and [has-property] [Individual].",
       "and [has-data-property] [literal].",
       "and [has-property] a [class].",
       "or [has-property] [Individual].",
       "or [has-data-property] [literal].",
       "or [has-property] a [class].",
       "that [has-property] [Individual].",
       "that [has-data-property] [literal].",
       "that [has-property] a [class]." },           // 15 CLISTOBJ
     { "[has-property] [Individual].",
       "[has-data-property] [literal].",
       "[has-property] a [class]." },                // 16 VPLISTLOOP
     { "No options" },                               // 17 AUXLIST
     { "[has-property] [Individual].",
       "[has-data-property] [literal].",
       "[has-property] a [class]." },                // 18 VPCHAINLOOP
     { "No options" },                               // 19 AUXCHAIN
     { "[Individual].",
       "[literal].",
       "a [class].",
       "only [class].",
       "exactly [integer] [class].",
       "at least [integer] [class].",
       "at most [integer] [class]." },               // 20 VPCHAIN
     { "No options" },                               // 21 ICHAIN
     { "and [has-property] [Individual].",
       "and [has-data-property] [literal].",
       "and [has-property] a [class]." },            // 22 ICHAINOBJ
     { "and [has-property] [Individual].",
       "and [has-data-property] [literal].",
       "and [has-property] a [class]." },            // 23 LCHAINOBJ
     { "[class]." },                                 // 24 CCHAIN
     { "that [has-property] [Individual].",
       "that [has-data-property] [literal].",
       "that [has-property] a [class]." },           // 25 CCHAINOBJ
     { "No options" },                               // 26 FINISH
     { "No options" },                               // 27 COMMENT
     { "No options" },                               // 28 METADATA
     { "No options" },                               // 29 METAVALUE
     { "[class] is a [class]." },                    // 30 DSUB
     { "is a [class]." },                            // 31 DSUBJECT
     { "a [class]." },                               // 32 DCOPULAR
     { "[class]." },                                 // 33 DCOMP
     { "[class]." },                                 // 34 DCOMPLEMENT
     { "as a [class]." },                            // 35 DEFINED
     { "a [class]." },                               // 36 DEFAS
     { "that [has-property] something is a [class].",
       "that something [has-property] is a [class]." }, // 37 PROPSUBJ
     { "[has-property] something is a [class].",
       "something [has-property] is a [class]." },   // 38 PROPTHAT
     { "something is a [class]." },                  // 39 DOMAINAUX
     { "something is a [class]." },                  // 40 DOMAIN
     { "No options" },                               // 41 DATADOMAIN
     { "is a [class]." },                            // 42 DOMAINVP
     { "[has-property] is a [class]." },             // 43 RANGE
     { "[has-property] is a [class]." },             // 44 RANGEAUX
     { "is a [class]." },                            // 45 RANGEVP
     { "a [class]." },                               // 46 PROPOBJ
     { "[class]." },                                 // 47 CPROP
     { "No options" },                               // 48 CPROPOBJ
     { "a [class].",
       "only [class].",
       "exactly [integer] [class].",
       "least [integer] [class].",
       "most [integer] [class]." },                  // 49 ATLIST
     { "[integer] [class]." },                       // 50 EQUALITYLIST
     { "[integer] [class]." },                       // 51 INEQUALITYLIST
     { "a [class].",
       "only [class].",
       "exactly [integer] [class].",
       "least [integer] [class].",
       "most [integer] [class]." },                  // 52 ATCHAIN
     { "[integer] [class]." },                       // 53 EQUALITYCHAIN
     { "[integer] [class]." },                       // 54 INEQUALITYCHAIN
     { "<[has-property]> is [attribute].",
       "<[has-property]> and <[has-property]> are disjoint.",
       "<[has-property]> and <[has-property]> are equivalent.",
       "<[has-property]> and <[has-property]> are inverse." },  // 55 PROPINTRO
     { "[has-property]> is [attribute]." },          // 56 OPENPROP
     { "No options" },                               // 57 AUXPROP
     { "> is [attribute]." },                        // 58 VPPROP
     { "is [attribute].",
       "is a sub-property of <[has-property]>.",
       "<and <[has-property]> are disjoint.",
       "<and <[has-property]> are equivalent.",
       "<and <[has-property]> are inverse." },       // 59 CLOSEPROP
     { "[attribute].",
       "a sub-property of <[has-property]>." },     // 60 PROPCOP
     { "No options" },                               // 61 PROPATTR
     { "<[has-data-property]> is functional." },     // 62 DATAPROPINTRO
     { "[has-data-property]> is functional." },      // 63 OPENDATAPROP
     { "No options" },                               // 64 AUXDATAPROP
     { "> is functional." },                         // 65 VPDATAPROP
     { "is functional.",
       "is a sub-property of <[has-data-property]>.",
       "<and <[has-data-property]> are disjoint.",
       "<and <[has-data-property]> are equivalent." }, // 66 CLOSEDATAPROP
     { "functional.",                              
       "a sub-property of <[has-data-property]>." }, // 67 DATAPROPCOP
     { "No options" },                               // 68 DATAPROPATTR
     { "<[has-property]> are disjoint.",
       "<[has-property]> are equivalent.",
       "<[has-property]> are inverse." },            // 69 PROPINTROREL
     { "[has-property]> are disjoint.",
       "[has-property]> are equivalent.",
       "[has-property]> are inverse." },             // 70 OPENPROPREL
     { "No options" },                               // 71 AUXPROPREL
     { "> are disjoint.",
       "> are equivalent.",
       "> are inverse." },                           // 72 VPPROPREL
     { "are disjoint.",
       "are equivalent.",
       "are inverse." },                             // 73 CLOSEPROPREL
     { "disjoint.",
       "equivalent.",
       "inverse." },                                 // 74 PROPCOPREL
     { "No options" },                               // 75 PROPATTRREL
     { "sub-property of <[has-property]>." },        // 76 PROPSUBA
     { "of <[has-property]>." },                     // 77 PROPSUBB
     { "<[has-property]>." },                        // 78 PROPINTROSUB
     { "[has-property]>." },                         // 79 OPENPROPSUB
     { "No options" },                               // 80 AUXPROPSUB
     { ">." },                                       // 81 VPPROPSUB
     { "No options" },                               // 82 CLOSEPROPSUB
     { "<[has-data-property]> are disjoint.",
       "<[has-data-property]> are equivalent." },    // 83 DATAPROPINTROREL
     { "[has-data-property]> are disjoint.",
       "[has-data-property]> are equivalent." },     // 84 OPENDATAPROPREL
     { "No options" },                               // 85 AUXDATAPROPREL
     { "> are disjoint.",
       "> are equivalent." },                        // 86 VPDATAPROPREL
     { "are disjoint.",
       "are equivalent." },                          // 87 CLOSEDATAPROPREL
     { "disjoint.",
       "equivalent." },                              // 88 DATAPROPCOPREL
     { "No options" },                               // 89 DATAPROPATTRREL
     { "sub-property of <[has-data-property]>." },   // 90 DATAPROPSUBA
     { "of <[has-data-property]>." },                // 91 DATAPROPSUBB
     { "<[has-data-property]>." },                   // 92 DATAPROPINTROSUB
     { "[has-data-property]>." },                    // 93 OPENDATAPROPSUB
     { "No options" },                               // 94 AUXDATAPROPSUB
     { ">." },                                       // 95 VPDATAPROPSUB
     { "No options" }                                // 96 CLOSEDATAPROPSUB
   };

// Associates options for completing/continuing the sentence with each state
  private static String continuations[] =
   { "Incorrect sentence",                                        // 0 REJECT
     "A|An|The|Every|No|Anything|PROPER-NAME",                    // 1 START
     "NOUN|ADJECTIVE|PREPOSITION|PROPER-NAME|NUMBER|STRING",      // 2 ISUB
     "NOUN|ADJECTIVE|PREPOSITION|PROPER-NAME|NUMBER|STRING",      // 3 CSUB
     "is|has|the|VERB|NOUN|ADJECTIVE|PREPOSITION|PROPER-NAME|NUMBER|STRING", // 4 SUBJECT
     "a|an|defined|NOUN|ADJECTIVE|PREPOSITION",                   // 5 COPULAR
     "NOUN|ADJECTIVE|PREPOSITION",                                // 6 AUX
     "NOUN|ADJECTIVE|PREPOSITION|PROPER-NAME|NUMBER|STRING",      // 7 NPLIST
     "and|or|that|the|NOUN|ADJECTIVE|PREPOSITION|PROPER-NAME|NUMBER|STRING|FULLSTOP", // 8 COMPLEMENT
     "a|an",                                                      // 9 NPLISTLOOP
     "a|an|the|only|at|exactly|PROPER-NAME|NOUN|ADJECTIVE|PREPOSITION|NUMBER|STRING", // 10 VPLIST
     "NOUN|ADJECTIVE|PREPOSITION|PROPER-NAME|NUMBER|STRING",      // 11 ILIST
     "and|or|the|NOUN|ADJECTIVE|PREPOSITION|PROPER-NAME|NUMBER|STRING|FULLSTOP", // 12 ILISTOBJ
     "and|or|FULLSTOP",                                              // 13 LLISTOBJ
     "NOUN|ADJECTIVE|PREPOSITION|NUMBER|STRING|PROPER-NAME",      // 14 CLIST
     "and|or|that|the|NOUN|ADJECTIVE|PREPOSITION|PROPER-NAME|FULLSTOP", // 15 CLISTOBJ
     "is|has|VERB",                                               // 16 VPLISTLOOP
     "NOUN|ADJECTIVE|PREPOSITION",                                // 17 AUXLIST
     "is|has|VERB",                                               // 18 VPCHAINLOOP
     "NOUN|ADJECTIVE|PREPOSITION",                                // 19 AUXCHAIN
     "a|an|the|only|at|exactly|PROPER-NAME|NOUN|ADJECTIVE|PREPOSITION|NUMBER|STRING", // 20 VPCHAIN
     "NOUN|ADJECTIVE|PREPOSITION|PROPER-NAME|NUMBER|STRING",      // 21 ICHAIN
     "the|NOUN|ADJECTIVE|PREPOSITION|PROPER-NAME|NUMBER|STRING|FULLSTOP", // 22 ICHAINOBJ
     "FULLSTOP",                                                  // 23 LCHAINOBJ
     "NOUN|ADJECTIVE|PREPOSITION|PROPER-NAME|NUMBER|STRING",      // 24 CCHAIN
     "that|the|NOUN|ADJECTIVE|PREPOSITION|PROPER-NAME|NUMBER|STRING|FULLSTOP", // 25 CCHAINOBJ
     "Complete sentence",                                         // 26 FINISH
     "Comment message",                                           // 27 COMMENT
     "OntologyIRI|Verb",                                          // 28 METADATA
     "For verb give third person singular",                       // 29 METAVALUE
     "NOUN|ADJECTIVE|PREPOSITION|PROPER-NAME|NUMBER|STRING",      // 30 DSUB
     "is|the|NOUN|ADJECTIVE|PREPOSITION|PROPER-NAME|NUMBER|STRING", // 31 DSUBJECT
     "a",                                                         // 32 DCOPULAR
     "NOUN|ADJECTIVE|PREPOSITION|PROPER-NAME|NUMBER|STRING",      // 33 DCOMP
     "the|NOUN|ADJECTIVE|PREPOSITION|PROPER-NAME|NUMBER|STRING",  // 34 DCOMPLEMENT
     "as|NOUN|ADJECTIVE|PREPOSITION",                             // 35 DEFINED
     "a",                                                         // 36 DEFAS
     "that",                                                      // 37 PROPSUBJ
     "something|is|has|VERB",                                     // 38 PROPTHAT
     "NOUN|ADJECTIVE|PREPOSITION",                                // 39 DOMAINAUX
     "some|something|NOUN|ADJECTIVE|PREPOSITION",                 // 40 DOMAIN
     "value",                                                     // 41 DATADOMAIN
     "is",                                                        // 42 DOMAINVP
     "is|has|VERB",                                               // 43 RANGE
     "NOUN|ADJECTIVE|PREPOSITION",                                // 44 RANGEAUX
     "is|NOUN|ADJECTIVE|PREPOSITION",                             // 45 RANGEVP
     "a",                                                         // 46 PROPOBJ
     "NOUN|ADJECTIVE|PREPOSITION|PROPER-NAME|NUMBER|STRING",      // 47 CPROP
     "NOUN|ADJECTIVE|PREPOSITION|PROPER-NAME|NUMBER|STRING|FULLSTOP", // 48 CPROPOBJ
     "a|only|exactly|most|least|PREPOSITION",                     // 49 ATLIST
     "INTEGER",                                                   // 50 EQUALITYLIST
     "INTEGER",                                                   // 51 INEQUALITYLIST
     "a|only|exactly|most|least|PREPOSITION",                     // 52 ATCHAIN
     "INTEGER",                                                   // 53 EQUALITYCHAIN
     "INTEGER",                                                   // 54 INEQUALITYCHAIN
     "<|NOUN|ADJECTIVE|PREPOSITION|PROPER-NAME|NUMBER|STRING",    // 55 PROPINTRO
     "is|has|VERB",                                               // 56 OPENPROP
     "NOUN|ADJECTIVE|PREPOSITION",                                // 57 AUXPROP
     ">|NOUN|ADJECTIVE|PREPOSITION",                              // 58 VPPROP
     "is|and",                                                    // 59 CLOSEPROP
     "a|ATTRIBUTE",                                               // 60 PROPCOP
     "FULLSTOP",                                                  // 61 PROPATTR
     "<|NOUN|ADJECTIVE|PREPOSITION|PROPER-NAME|NUMBER|STRING",    // 62 DATAPROPINTRO
     "is|has|VERB",                                               // 63 OPENDATAPROP
     "NOUN|ADJECTIVE|PREPOSITION",                                // 64 AUXDATAPROP
     ">|NOUN|ADJECTIVE|PREPOSITION",                              // 65 VPDATAPROP
     "is",                                                        // 66 CLOSEDATAPROP
     "functional",                                                // 67 DATAPROPCOP
     "FULLSTOP",                                                  // 68 DATAPROPATTR
     "<",                                                         // 69 PROPINTROREL
     "is|has|VERB",                                               // 70 OPENPROPREL
     "NOUN|ADJECTIVE|PREPOSITION",                                // 71 AUXPROPREL
     ">|NOUN|ADJECTIVE|PREPOSITION",                              // 72 VPPROPREL
     "are",                                                       // 73 CLOSEPROPREL
     "disjoint|equivalent|inverse",                               // 74 PROPCOPREL
     "FULLSTOP",                                                  // 75 PROPATTRREL
     "sub-property",                                              // 76 PROPSUBA
     "of",                                                        // 77 PROPSUBB
     "<",                                                         // 78 PROPINTROSUB
     "is|has|VERB",                                               // 79 OPENPROPSUB
     "NOUN|ADJECTIVE|PREPOSITION",                                // 80 AUXPROPSUB
     ">|NOUN|ADJECTIVE|PREPOSITION",                              // 81 VPPROPSUB
     "FULLSTOP",                                                  // 82 CLOSEPROPSUB
     "<",                                                         // 83 DATAPROPINTROREL
     "is|has|VERB",                                               // 84 OPENDATAPROPREL
     "NOUN|ADJECTIVE|PREPOSITION",                                // 85 AUXDATAPROPREL
     ">|NOUN|ADJECTIVE|PREPOSITION",                              // 86 VPDATAPROPREL
     "are",                                                       // 87 CLOSEDATAPROPREL
     "disjoint|equivalent",                                       // 88 DATAPROPCOPREL
     "FULLSTOP",                                                  // 89 DATAPROPATTRREL
     "sub-property",                                              // 90 DATAPROPSUBA
     "of",                                                        // 91 DATAPROPSUBB
     "<",                                                         // 92 DATAPROPINTROSUB
     "is|has|VERB",                                               // 93 OPENDATAPROPSUB
     "NOUN|ADJECTIVE|PREPOSITION",                                // 94 AUXDATAPROPSUB
     ">|NOUN|ADJECTIVE|PREPOSITION",                              // 95 VPDATAPROPSUB
     "FULLSTOP"                                                   // 96 CLOSEDATAPROPSUB
   };

// Test cases
  private static String test[] =
    { "The queen is a head of state.",
      "A white van driver is a driver and a menace to traffic.",
      "The queen is married to the Duke of Edinburgh.",
      "A van driver drives a van.",
      "The Duke of Edinburgh has as residence Buckingham Palace and is married to a monarch.",
      "The queen is a monarch and a head of state that is married to the Duke of Edinburgh.",
      "Elizabeth is an octogenerian and a queen that is married to a duke and has as son Prince Charles.",
      "Elizabeth is an octogenerian and a queen that has as son Prince Charles and is married to a duke that lives in a palace that is located in a city that is capital of the United Kingdom.",
      "% This is a comment",
      "# OntologyIRI http://ontology.org/23a",
      "# VERB takes",
      "No man is an island.",
      "A spouse is defined as a person that is married to a person.",
      "The property <is father of> is functional.",
      "The properties <is father of> and <is mother of> are disjoint.",
      "The property <is father of> is a sub-property of <is parent of>.",
      "The data-properties <is aged> and <is named> are disjoint.",
      "The data-property <is nicknamed> is a sub-property of <is named>."
    };

// Main method used only for testing
  public static void main(String[] args)
  { Sentence sentence;
    Vector<Verb> verbs = new Vector<Verb>();
    Verb v = new Verb("drive","drives");
    verbs.addElement(v);
    v = new Verb("live","lives");
    verbs.addElement(v);
    for (int i=0; i<test.length; i++)
    { sentence = new Sentence(0,test[i],0,verbs);
      System.out.println(test[i]);
      System.out.println(sentence.getOwlFunctionalSyntax());
    }
    int r = getRole(START,SUBJECT);
    System.out.println(START + "-" + SUBJECT + " = " + r); 
  }

// Constructor
  public Sentence(int arg1, String arg2, int arg3, Vector<Verb> arg4)
  { sentenceNumber = arg1;
    textString = arg2;
    startPosition = arg3;
    verbs = arg4;
    numberOfCharacters = textString.length();
    endPosition = startPosition + numberOfCharacters;
    makeWords();
    axiom = new OwlTerm(OwlTerm.UNSPECIFIED,2);
    correctSentence = false;
    transduce();
  }

  public void setTextString(String stringArg) {textString = stringArg; }
  public void setStartPosition(int intArg) {startPosition = intArg; }

  public String getTextString() {return textString; }
  public int getSentenceNumber() {return sentenceNumber; }
  public int getStartPosition() {return startPosition; }
  public int getEndPosition() {return endPosition; }
  public OwlTerm getOwlTerm() {return axiom; }
  public boolean sentenceIsCorrect() {return correctSentence; }
  public Word[] getWords() {return word; }

  public String getOwlFunctionalSyntax()
  { return axiom.getPrintForm();
  }

// Keep original line for comment or metadata or incorrect sentence
  public String getGeneratedSentence()
  { String s = "";
    if ((correctSentence == true) && (isCommentOrMetadata(axiom) == false))
    { OwlEnglish english = new OwlEnglish(axiom);
      s = english.getTextString();
    }
    else s = textString; 
    return s;
  }

// Should be called only if current word is in this sentence.
// If it is not in the sentence, defaults to the first word.
  public Word getCurrentWord(int caretPosition)
  { int wordNumber = 0;
    for (int i=0; i<word.length; i++)
    { if ((word[i].getWordStart() <= caretPosition)
          && (word[i].getWordFinish() >= caretPosition))
      wordNumber = i;
    }
    return word[wordNumber];
  }  

// Return a message associated with the transition of the last completed word
  public String getMessage()
  { String message = "Starting a sentence";
    if ((numberOfWords == 1) && (textString.length() > 1))
     { if (textString.charAt(textString.length()-1) == ' ')
       message = wordMessage(0);
     }
    if (numberOfWords > 1)
    { if (word[word.length-1].getWordString().equals("."))
      message = wordMessage(word.length-1);
      else message = wordMessage(word.length-2);
    }
    return message;  
  }

// Return exit state of the last completed word
  public int getCurrentState()
  { int state = START;
    if ((numberOfWords == 1) && (textString.length() > 1))
     { if (textString.charAt(textString.length()-1) == ' ')
       state = word[0].getWordExitState();
     }
    if (numberOfWords > 1)
    { if (word[word.length-1].getWordString().equals("."))
      state = word[word.length-1].getWordExitState();
      else state = word[word.length-2].getWordExitState();
    }
    return state;  
  }

// Return options associated with current state

  public String[] getOptions()
  { int cstate = getCurrentState();
    return filterOptions(options[cstate]);
  }

// Return continuations associated with current state
  public String getContinuations()
  { int cstate = getCurrentState();
    return continuations[cstate];
  }

  private String[] filterOptions(String[] options)
  { Vector<String> vector = new Vector<String>();
    for (int i=0; i<options.length; i++)
    if (relevantOption(options[i]) == true) vector.addElement(options[i]);
    String[] output = new String[vector.size()];
    vector.toArray(output);
    return output;
  }

  private boolean relevantOption(String s)
  { boolean tval = true;
    if ((listType == INTERSECTION) && s.startsWith("or")) tval = false;
    if ((listType == UNION) && (s.startsWith("and") || s.startsWith("that"))) tval = false;
    return tval;
  }

  private String wordMessage(int w)
  { int transition = 0;
    int entry = word[w].getWordEntryState();
    int exit = word[w].getWordExitState();
    for (int i=0; i<role.length; i++)
    if ((role[i][0] == entry) && (role[i][1] == exit)) transition = i+1;
    return transitionMessage[transition];
  }

  public static int getRole(int entry, int exit)
  { int r = ERROR;
    for (int i=0; i<role.length; i++)
    if ((role[i][0] == entry) && (role[i][1] == exit)) r = role[i][2];
    return r;
  }

// The Word constructor will assign a syntactic category to each word.
// State and Role variables on each word are assigned by the transducer.
// The vector of user-defined verbs is provided as final argument
  private void makeWords()
  { Tokeniser tokeniser = new Tokeniser(textString);
    numberOfWords = tokeniser.getNumberOfTokens();
    word = new Word[numberOfWords];
    for (int i=0; i<numberOfWords; i++)
    { String istring = tokeniser.getTokenString(i);
      int istart = tokeniser.getTokenStart(i) + startPosition;
      int ifinish = tokeniser.getTokenFinish(i) + startPosition;
      word[i] = new Word(i,istring,istart,ifinish,verbs);
    }
  }

// Assign each word in the sentence the state in the network that is
// reached by traversing it (or REJECT if the word violates the grammar).

// At the same time build the OwlTerm interpretation of the sentence in
// axiom (OwlTerm for whole sentence) and register (OwlTerm constituent
// currently under construction).

  private void transduce()
  { int currentState = START;
    int lastState;
    int currentNumber = OwlEnglish.SINGULAR;
    OwlTerm register = axiom;
    OwlTerm newTerm, newArg;
    String namepart = "";
    listType = UNSPECIFIED;
    for (int i=0; i<numberOfWords; i++)
    { lastState = currentState;
//      String sss = word[i].getWordString();
//      System.out.println(currentState + " " + sss);
      if ((currentState == START) && word[i].getWordString().equals("%"))
      { currentState = COMMENT;
        axiom = new OwlTerm(OwlTerm.COMMENT,0);
        register = axiom;
        register.setEntityName("");
      }
      else if ((currentState == START) && word[i].getWordString().equals("#"))
      { currentState = METADATA;
        axiom = new OwlTerm(OwlTerm.METADATA,0);
        register = axiom;
        register.setEntityName("");
      }
      else if ((currentState == START) && quantifierAnything(word[i]))
      { currentState = PROPSUBJ;
      }
      else if ((currentState == START) && definiteArticle(word[i]))
      { currentState = ISUB;
        register.setFunctor(OwlTerm.CLASS_ASSERTION);
        newTerm = new OwlTerm(OwlTerm.NAMED_INDIVIDUAL,0);
        newTerm.addWord(word[i]);
        register.setArgument(0,newTerm);
      }
      else if ((currentState == START) && properName(word[i]))
      { currentState = SUBJECT;
        register.setFunctor(OwlTerm.CLASS_ASSERTION);
        newTerm = new OwlTerm(OwlTerm.NAMED_INDIVIDUAL,0);
        newTerm.addWord(word[i]);
        register.setArgument(0,newTerm);
      }
      else if ((currentState == START) && determinerEvery(word[i]))
      { currentState = CSUB;
        register.setFunctor(OwlTerm.SUB_CLASS_OF);
        newTerm = new OwlTerm(OwlTerm.CLASS,0);
        register.setArgument(0,newTerm);
      }
      else if ((currentState == START) && determinerNo(word[i]))
      { currentState = DSUB;
        register.setFunctor(OwlTerm.DISJOINT_CLASSES);
        newTerm = new OwlTerm(OwlTerm.CLASS,0);
        register.setArgument(0,newTerm);
      }
      else if ((currentState == PROPSUBJ) && conjunctionThat(word[i]))
      { currentState = PROPTHAT;
      }
      else if ((currentState == PROPTHAT) && quantifierSomething(word[i]))
      { currentState = RANGE;
        register.setFunctor(OwlTerm.OBJECT_PROPERTY_RANGE);
        newTerm = new OwlTerm(OwlTerm.OBJECT_PROPERTY,0);
        register.setArgument(0,newTerm);
      }
      else if ((currentState == PROPTHAT) && verbPhraseOpening12(word[i]))
      { currentState = DOMAINAUX;
        register.setFunctor(OwlTerm.OBJECT_PROPERTY_DOMAIN);
        newTerm = new OwlTerm(OwlTerm.OBJECT_PROPERTY,0);
        newTerm.addWord(word[i]);
        register.setArgument(0,newTerm);
      }
      else if ((currentState == PROPTHAT) && verbPhraseOpening3(word[i]))
      { currentState = DOMAIN;
        register.setFunctor(OwlTerm.OBJECT_PROPERTY_DOMAIN);
        newTerm = new OwlTerm(OwlTerm.OBJECT_PROPERTY,0);
        newTerm.addWord(word[i]);
        register.setArgument(0,newTerm);
      }
      else if ((currentState == RANGE) && verbPhraseOpening12(word[i]))
      { currentState = RANGEAUX;
        newArg = register.getArgument(0);
        newArg.addWord(word[i]);
      }
      else if ((currentState == RANGE) && verbPhraseOpening3(word[i]))
      { currentState = RANGEVP;
        newArg = register.getArgument(0);
        newArg.addWord(word[i]);
      }
      else if ((currentState == DOMAINAUX) && verbPhraseContinuation(word[i]))
      { currentState = DOMAIN;
        newArg = register.getArgument(0);
        namepart = newArg.getEntityName();
        namepart = namepart + " " + word[i].getWordString();
        newArg.addWord(word[i]);
      }
      else if ((currentState == RANGEAUX) && verbPhraseContinuation(word[i]))
      { currentState = RANGEVP;
        newArg = register.getArgument(0);
        newArg.addWord(word[i]);
      }
      else if ((currentState == DOMAIN) && verbPhraseContinuation(word[i]))
      { currentState = DOMAIN;
        newArg = register.getArgument(0);
        newArg.addWord(word[i]);
      }
      else if ((currentState == RANGEVP) && verbPhraseContinuation(word[i]))
      { currentState = RANGEVP;
        newArg = register.getArgument(0);
        newArg.addWord(word[i]);
      }
      else if ((currentState == DOMAIN) && quantifierSomething(word[i]))
      { currentState = DOMAINVP;
      }
      else if ((currentState == DOMAIN) && quantifierSome(word[i]))
      { currentState = DATADOMAIN;
        register.setFunctor(OwlTerm.DATA_PROPERTY_DOMAIN);
        register.getArgument(0).setFunctor(OwlTerm.DATA_PROPERTY);
      }
      else if ((currentState == DATADOMAIN) && keyWord(word[i],"value"))
      { currentState = DOMAINVP;
      }
      else if ((currentState == RANGEVP) && verbPhraseOpening1(word[i]))
      { currentState = PROPOBJ;
      }
      else if ((currentState == DOMAINVP) && verbPhraseOpening1(word[i]))
      { currentState = PROPOBJ;
      }
      else if ((currentState == PROPOBJ) && indefiniteArticle(word[i]))
      { currentState = CPROP;
      }
      else if ((currentState == CPROP) && nounPhraseContinuation(word[i]))
      { currentState = CPROPOBJ;
        newTerm = new OwlTerm(OwlTerm.CLASS,0);
        newTerm.addWord(word[i]);
        register.setArgument(1,newTerm);
      }
      else if ((currentState == CPROPOBJ) && nounPhraseContinuation(word[i]))
      { currentState = CPROPOBJ;
        newArg = register.getArgument(1);
        newArg.addWord(word[i]);
      }
      else if ((currentState == CPROPOBJ) && fullStop(word[i]))
      { currentState = FINISH;
      }
      else if ((currentState == ISUB) &&
               (keyWord(word[i],"property") || keyWord(word[i],"properties")))
      { currentState = PROPINTRO;
        newArg = register.getArgument(0);
        newArg.addWord(word[i]);
      }
      else if ((currentState == ISUB) &&
               (keyWord(word[i],"data-property") || keyWord(word[i],"data-properties")))
      { currentState = DATAPROPINTRO;
        newArg = register.getArgument(0);
        newArg.addWord(word[i]);
      }
      else if ((currentState == ISUB) && nounPhraseContinuation(word[i]))
      { currentState = SUBJECT;
        newArg = register.getArgument(0);
        newArg.addWord(word[i]);
      }
      else if ((currentState == CSUB) && nounPhraseContinuation(word[i]))
      { currentState = SUBJECT;
        newArg = register.getArgument(0);
        newArg.addWord(word[i]);
      }
      else if ((currentState == DSUB) && nounPhraseContinuation(word[i]))
      { currentState = DSUBJECT;
        newArg = register.getArgument(0);
        newArg.addWord(word[i]);
      }
      else if ((currentState == SUBJECT) && nounPhraseContinuation(word[i]))
      { currentState = SUBJECT;
        newArg = register.getArgument(0);
        newArg.addWord(word[i]);
      }
      else if ((currentState == SUBJECT) && verbPhraseOpening1(word[i]))
      { currentState = COPULAR;
      }
      else if ((currentState == DSUBJECT) && nounPhraseContinuation(word[i]))
      { currentState = DSUBJECT;
        newArg = register.getArgument(0);
        newArg.addWord(word[i]);
      }
      else if ((currentState == DSUBJECT) && verbPhraseOpening1(word[i]))
      { currentState = DCOPULAR;
      }
      else if ((currentState == PROPINTRO) && nounPhraseContinuation(word[i]))
      { currentState = SUBJECT;
        newArg = register.getArgument(0);
        newArg.addWord(word[i]);
      }
      else if ((currentState == PROPINTRO) && openAngleBracket(word[i]))
      { currentState = OPENPROP;
        axiom = new OwlTerm(OwlTerm.UNSPECIFIED,1);
        register = axiom;
      }
      else if ((currentState == OPENPROP) && verbPhraseOpening12(word[i]))
      { currentState = AUXPROP;
        newTerm = new OwlTerm(OwlTerm.OBJECT_PROPERTY,0);
        newTerm.addWord(word[i]);
        register.setArgument(0,newTerm);
      }
      else if ((currentState == OPENPROP) && verbPhraseOpening3(word[i]))
      { currentState = VPPROP;
        newTerm = new OwlTerm(OwlTerm.OBJECT_PROPERTY,0);
        newTerm.addWord(word[i]);
        register.setArgument(0,newTerm);
      }
      else if ((currentState == AUXPROP) && verbPhraseContinuation(word[i]))
      { currentState = VPPROP;
        newArg = register.getArgument(0);
        newArg.addWord(word[i]);
      }
      else if ((currentState == VPPROP) && verbPhraseContinuation(word[i]))
      { currentState = VPPROP;
        newArg = register.getArgument(0);
        newArg.addWord(word[i]);
      }
      else if ((currentState == VPPROP) && closeAngleBracket(word[i]))
      { currentState = CLOSEPROP;
      }
      else if ((currentState == CLOSEPROP) && verbPhraseOpening1(word[i]))
      { currentState = PROPCOP;
      }
      else if ((currentState == CLOSEPROP) && conjunctionAnd(word[i]))
      { currentState = PROPINTROREL;
      }
      else if ((currentState == PROPINTROREL) && openAngleBracket(word[i]))
      { currentState = OPENPROPREL;
        newArg = register.getArgument(0);
        axiom = new OwlTerm(OwlTerm.UNSPECIFIED,2);
        axiom.setArgument(0,newArg);
        register = axiom;
      }
      else if ((currentState == OPENPROPREL) && verbPhraseOpening12(word[i]))
      { currentState = AUXPROPREL;
        newTerm = new OwlTerm(OwlTerm.OBJECT_PROPERTY,0);
        newTerm.addWord(word[i]);
        register.setArgument(1,newTerm);
      }
      else if ((currentState == OPENPROPREL) && verbPhraseOpening3(word[i]))
      { currentState = VPPROPREL;
        newTerm = new OwlTerm(OwlTerm.OBJECT_PROPERTY,0);
        newTerm.addWord(word[i]);
        register.setArgument(1,newTerm);
      }
      else if ((currentState == AUXPROPREL) && verbPhraseContinuation(word[i]))
      { currentState = VPPROPREL;
        newArg = register.getArgument(1);
        newArg.addWord(word[i]);
      }
      else if ((currentState == VPPROPREL) && verbPhraseContinuation(word[i]))
      { currentState = VPPROPREL;
        newArg = register.getArgument(1);
        newArg.addWord(word[i]);
      }
      else if ((currentState == VPPROPREL) && closeAngleBracket(word[i]))
      { currentState = CLOSEPROPREL;
      }
      else if ((currentState == CLOSEPROPREL) && verbPhraseOpening1(word[i]))
      { currentState = PROPCOPREL;
      }
      else if ((currentState == PROPCOPREL) && propertyRelation(word[i]))
      { currentState = PROPATTRREL;
        register.setFunctor(relationFunctor(word[i]));
      }
      else if ((currentState == PROPATTRREL) && fullStop(word[i]))
      { currentState = FINISH;
      }
      else if ((currentState == PROPCOP) && propertyAttribute(word[i]))
      { currentState = PROPATTR;
        register.setFunctor(attributeFunctor(word[i]));
      }
      else if ((currentState == PROPCOP) && indefiniteArticle(word[i]))
      { currentState = PROPSUBA;
      }
      else if ((currentState == PROPSUBA) && keyWord(word[i],"sub-property"))
      { currentState = PROPSUBB;
        newArg = register.getArgument(0);
        axiom = new OwlTerm(OwlTerm.SUB_OBJECT_PROPERTY_OF,2);
        axiom.setArgument(0,newArg);
        register = axiom;
      }
      else if ((currentState == PROPSUBB) && keyWord(word[i],"of"))
      { currentState = PROPINTROSUB;
      }
      else if ((currentState == PROPINTROSUB) && openAngleBracket(word[i]))
      { currentState = OPENPROPSUB;
      }
      else if ((currentState == OPENPROPSUB) && verbPhraseOpening12(word[i]))
      { currentState = AUXPROPSUB;
        newTerm = new OwlTerm(OwlTerm.OBJECT_PROPERTY,0);
        newTerm.addWord(word[i]);
        register.setArgument(1,newTerm);
      }
      else if ((currentState == OPENPROPSUB) && verbPhraseOpening3(word[i]))
      { currentState = VPPROPSUB;
        newTerm = new OwlTerm(OwlTerm.OBJECT_PROPERTY,0);
        newTerm.addWord(word[i]);
        register.setArgument(1,newTerm);
      }
      else if ((currentState == AUXPROPSUB) && verbPhraseContinuation(word[i]))
      { currentState = VPPROPSUB;
        newArg = register.getArgument(1);
        newArg.addWord(word[i]);
      }
      else if ((currentState == VPPROPSUB) && verbPhraseContinuation(word[i]))
      { currentState = VPPROPSUB;
        newArg = register.getArgument(1);
        newArg.addWord(word[i]);
      }
      else if ((currentState == VPPROPSUB) && closeAngleBracket(word[i]))
      { currentState = CLOSEPROPSUB;
      }
      else if ((currentState == CLOSEPROPSUB) && fullStop(word[i]))
      { currentState = FINISH;
      }
      else if ((currentState == DATAPROPINTROREL) && openAngleBracket(word[i]))
      { currentState = OPENDATAPROPREL;
        newArg = register.getArgument(0);
        axiom = new OwlTerm(OwlTerm.UNSPECIFIED,2);
        axiom.setArgument(0,newArg);
        register = axiom;
      }
      else if ((currentState == OPENDATAPROPREL) && verbPhraseOpening12(word[i]))
      { currentState = AUXDATAPROPREL;
        newTerm = new OwlTerm(OwlTerm.DATA_PROPERTY,0);
        newTerm.addWord(word[i]);
        register.setArgument(1,newTerm);
      }
      else if ((currentState == OPENDATAPROPREL) && verbPhraseOpening3(word[i]))
      { currentState = VPDATAPROPREL;
        newTerm = new OwlTerm(OwlTerm.DATA_PROPERTY,0);
        newTerm.addWord(word[i]);
        register.setArgument(1,newTerm);
      }
      else if ((currentState == AUXDATAPROPREL) && verbPhraseContinuation(word[i]))
      { currentState = VPDATAPROPREL;
        newArg = register.getArgument(1);
        newArg.addWord(word[i]);
      }
      else if ((currentState == VPDATAPROPREL) && verbPhraseContinuation(word[i]))
      { currentState = VPDATAPROPREL;
        newArg = register.getArgument(1);
        newArg.addWord(word[i]);
      }
      else if ((currentState == VPDATAPROPREL) && closeAngleBracket(word[i]))
      { currentState = CLOSEDATAPROPREL;
      }
      else if ((currentState == CLOSEDATAPROPREL) && verbPhraseOpening1(word[i]))
      { currentState = DATAPROPCOPREL;
      }
      else if ((currentState == DATAPROPCOPREL) && dataPropertyRelation(word[i]))
      { currentState = DATAPROPATTRREL;
        register.setFunctor(dataRelationFunctor(word[i]));
      }
      else if ((currentState == DATAPROPATTRREL) && fullStop(word[i]))
      { currentState = FINISH;
      }
      else if ((currentState == DATAPROPSUBA) && keyWord(word[i],"sub-property"))
      { currentState = DATAPROPSUBB;
        newArg = register.getArgument(0);
        axiom = new OwlTerm(OwlTerm.SUB_DATA_PROPERTY_OF,2);
        axiom.setArgument(0,newArg);
        register = axiom;
      }
      else if ((currentState == DATAPROPSUBB) && keyWord(word[i],"of"))
      { currentState = DATAPROPINTROSUB;
      }
      else if ((currentState == DATAPROPINTROSUB) && openAngleBracket(word[i]))
      { currentState = OPENDATAPROPSUB;
      }
      else if ((currentState == OPENDATAPROPSUB) && verbPhraseOpening12(word[i]))
      { currentState = AUXDATAPROPSUB;
        newTerm = new OwlTerm(OwlTerm.DATA_PROPERTY,0);
        newTerm.addWord(word[i]);
        register.setArgument(1,newTerm);
      }
      else if ((currentState == OPENDATAPROPSUB) && verbPhraseOpening3(word[i]))
      { currentState = VPDATAPROPSUB;
        newTerm = new OwlTerm(OwlTerm.DATA_PROPERTY,0);
        newTerm.addWord(word[i]);
        register.setArgument(1,newTerm);
      }
      else if ((currentState == AUXDATAPROPSUB) && verbPhraseContinuation(word[i]))
      { currentState = VPDATAPROPSUB;
        newArg = register.getArgument(1);
        newArg.addWord(word[i]);
      }
      else if ((currentState == VPDATAPROPSUB) && verbPhraseContinuation(word[i]))
      { currentState = VPDATAPROPSUB;
        newArg = register.getArgument(1);
        newArg.addWord(word[i]);
      }
      else if ((currentState == VPDATAPROPSUB) && closeAngleBracket(word[i]))
      { currentState = CLOSEDATAPROPSUB;
      }
      else if ((currentState == CLOSEDATAPROPSUB) && fullStop(word[i]))
      { currentState = FINISH;
      }
      else if ((currentState == PROPATTR) && fullStop(word[i]))
      { currentState = FINISH;
      }
      else if ((currentState == DATAPROPINTRO) && nounPhraseContinuation(word[i]))
      { currentState = SUBJECT;
        newArg = register.getArgument(0);
        newArg.addWord(word[i]);
      }
      else if ((currentState == DATAPROPINTRO) && openAngleBracket(word[i]))
      { currentState = OPENDATAPROP;
        axiom = new OwlTerm(OwlTerm.UNSPECIFIED,1);
        register = axiom;
      }
      else if ((currentState == OPENDATAPROP) && verbPhraseOpening12(word[i]))
      { currentState = AUXDATAPROP;
        newTerm = new OwlTerm(OwlTerm.DATA_PROPERTY,0);
        newTerm.addWord(word[i]);
        register.setArgument(0,newTerm);
      }
      else if ((currentState == OPENDATAPROP) && verbPhraseOpening3(word[i]))
      { currentState = VPDATAPROP;
        newTerm = new OwlTerm(OwlTerm.DATA_PROPERTY,0);
        newTerm.addWord(word[i]);
        register.setArgument(0,newTerm);
      }
      else if ((currentState == AUXDATAPROP) && verbPhraseContinuation(word[i]))
      { currentState = VPDATAPROP;
        newArg = register.getArgument(0);
        newArg.addWord(word[i]);
      }
      else if ((currentState == VPDATAPROP) && verbPhraseContinuation(word[i]))
      { currentState = VPDATAPROP;
        newArg = register.getArgument(0);
        newArg.addWord(word[i]);
      }
      else if ((currentState == VPDATAPROP) && closeAngleBracket(word[i]))
      { currentState = CLOSEDATAPROP;
      }
      else if ((currentState == CLOSEDATAPROP) && verbPhraseOpening1(word[i]))
      { currentState = DATAPROPCOP;
      }
      else if ((currentState == CLOSEDATAPROP) && conjunctionAnd(word[i]))
      { currentState = DATAPROPINTROREL;
      }
      else if ((currentState == DATAPROPCOP) && keyWord(word[i],"functional"))
      { currentState = DATAPROPATTR;
        register.setFunctor(OwlTerm.FUNCTIONAL_DATA_PROPERTY);
      }
      else if ((currentState == DATAPROPCOP) && indefiniteArticle(word[i]))
      { currentState = DATAPROPSUBA;
      }
      else if ((currentState == DATAPROPATTR) && fullStop(word[i]))
      { currentState = FINISH;
      }
      else if ((currentState == COPULAR) && indefiniteArticle(word[i]))
      { currentState = NPLIST;
        newTerm = new OwlTerm(OwlTerm.CLASS,0);
        register.setArgument(1,newTerm);
      }
      else if ((currentState == COPULAR) && keyWord(word[i],"defined"))
      { currentState = DEFINED;
        register.setFunctor(OwlTerm.EQUIVALENT_CLASSES);
      }
      else if ((currentState == DEFINED) && keyWord(word[i],"as"))
      { currentState = DEFAS;
      }
      else if ((currentState == DEFINED) && verbPhraseContinuation(word[i]))
      { currentState = VPLIST;
        register.setFunctor(OwlTerm.SUB_CLASS_OF);
        newTerm = new OwlTerm(OwlTerm.UNSPECIFIED,2);
        newArg = new OwlTerm(OwlTerm.OBJECT_PROPERTY,0);
        newArg.addWord(word[i-2]); // incorporate "is" into the property name
        newArg.addWord(word[i-1]); // incorporate "defined" into the property name
        newArg.addWord(word[i]);
        newTerm.setArgument(0,newArg);
        register.setArgument(1,newTerm);        
      }
      else if ((currentState == DEFAS) && indefiniteArticle(word[i]))
      { currentState = NPLIST;
        newTerm = new OwlTerm(OwlTerm.CLASS,0);
        register.setArgument(1,newTerm);
      }
      else if ((currentState == DCOPULAR) && indefiniteArticle(word[i]))
      { currentState = DCOMP;
        newTerm = new OwlTerm(OwlTerm.CLASS,0);
        register.setArgument(1,newTerm);
      }
      else if ((currentState == DCOMP) && nounPhraseContinuation(word[i]))
      { currentState = DCOMPLEMENT;
        newArg = register.getArgument(1);
        newArg.addWord(word[i]);
      }
      else if ((currentState == NPLIST) && nounPhraseContinuation(word[i]))
      { currentState = COMPLEMENT;
        newArg = register.getArgument(1);
        newArg.addWord(word[i]);
      }
      else if ((currentState == COMPLEMENT) && nounPhraseContinuation(word[i]))
      { currentState = COMPLEMENT;
        newArg = register.getArgument(1);
        newArg.addWord(word[i]);
      }
      else if ((currentState == DCOMPLEMENT) && nounPhraseContinuation(word[i]))
      { currentState = DCOMPLEMENT;
        newArg = register.getArgument(1);
        newArg.addWord(word[i]);
      }
      else if ((currentState == COMPLEMENT) && fullStop(word[i]))
      { currentState = FINISH;
      }
      else if ((currentState == DCOMPLEMENT) && fullStop(word[i]))
      { currentState = FINISH;
      }
      else if ((currentState == COMPLEMENT) && conjunctionAnd(word[i]) && (listType != UNION))
      { currentState = NPLISTLOOP;
        listType = INTERSECTION;
        newTerm = new OwlTerm(OwlTerm.OBJECT_INTERSECTION_OF,2);
        newTerm.setArgument(0,register.getArgument(1));
        register.setArgument(1,newTerm);
        register = newTerm;
      }
      else if ((currentState == COMPLEMENT) && conjunctionOr(word[i]) && (listType != INTERSECTION))
      { currentState = NPLISTLOOP;
        listType = UNION;
        newTerm = new OwlTerm(OwlTerm.OBJECT_UNION_OF,2);
        newTerm.setArgument(0,register.getArgument(1));
        register.setArgument(1,newTerm);
        register = newTerm;
      }
      else if ((currentState == NPLISTLOOP) && indefiniteArticle(word[i]))
      { currentState = NPLIST;
        newTerm = new OwlTerm(OwlTerm.CLASS,0);
        register.setArgument(1,newTerm);
      }
      else if ((currentState == COPULAR) && verbPhraseContinuation(word[i]))
      { currentState = VPLIST;
        newTerm = new OwlTerm(OwlTerm.UNSPECIFIED,2);
        newArg = new OwlTerm(OwlTerm.OBJECT_PROPERTY,0);
        newArg.addWord(word[i-1]); // incorporate "is" into the property name
        newArg.addWord(word[i]);
        newTerm.setArgument(0,newArg);
        register.setArgument(1,newTerm);
      }
      else if ((currentState == SUBJECT) && verbPhraseOpening2(word[i]))
      { currentState = AUX;
      }
      else if ((currentState == AUX) && verbPhraseContinuation(word[i]))
      { currentState = VPLIST;
        newTerm = new OwlTerm(OwlTerm.UNSPECIFIED,2);
        newArg = new OwlTerm(OwlTerm.OBJECT_PROPERTY,0);
        newArg.addWord(word[i-1]); // incorporate "has" into the property name
        newArg.addWord(word[i]);
        newTerm.setArgument(0,newArg);
        register.setArgument(1,newTerm);
      }
      else if ((currentState == SUBJECT) && verbPhraseOpening3(word[i]))
      { currentState = VPLIST;
        newTerm = new OwlTerm(OwlTerm.UNSPECIFIED,2);
        newArg = new OwlTerm(OwlTerm.OBJECT_PROPERTY,0);
        newArg.addWord(word[i]);
        newTerm.setArgument(0,newArg);
        register.setArgument(1,newTerm);
      }
      else if ((currentState == VPLIST) && keyWord(word[i],"at"))
// Could be start of quantifier or continuation of property name, so suspend judgement
      { currentState = ATLIST;
      }
      else if ((currentState == VPLIST) && quantifierExactly(word[i]))
      { currentState = EQUALITYLIST;
        newTerm = register.getArgument(1);
        newTerm.setFunctor(OwlTerm.OBJECT_EXACT_CARDINALITY);
        newArg = new OwlTerm(OwlTerm.CLASS,0);
        newTerm.setArgument(1,newArg);
      }
      else if ((currentState == VPLIST) && quantifierOnly(word[i]))
      { currentState = CLIST;
        newTerm = register.getArgument(1);
        newTerm.setFunctor(OwlTerm.OBJECT_ALL_VALUES_FROM);
        newArg = new OwlTerm(OwlTerm.CLASS,0);
        newTerm.setArgument(1,newArg);
        currentNumber = OwlEnglish.PLURAL;
      }
      else if ((currentState == VPLIST) && indefiniteArticle(word[i]))
      { currentState = CLIST;
        newTerm = register.getArgument(1);
        newTerm.setFunctor(OwlTerm.OBJECT_SOME_VALUES_FROM);
        newArg = new OwlTerm(OwlTerm.CLASS,0);
        newTerm.setArgument(1,newArg);
        currentNumber = OwlEnglish.SINGULAR;
      }
      else if ((currentState == VPLIST) && verbPhraseContinuation(word[i]))
      { currentState = VPLIST;
        newTerm = register.getArgument(1);
        newArg = newTerm.getArgument(0);
        newArg.addWord(word[i]);
      }
      else if ((currentState == VPLIST) && definiteArticle(word[i]))
      { currentState = ILIST;
        newTerm = register.getArgument(1);
        newTerm.setFunctor(OwlTerm.OBJECT_HAS_VALUE);
        newArg = new OwlTerm(OwlTerm.NAMED_INDIVIDUAL,0);
        newArg.addWord(word[i]);
        newTerm.setArgument(1,newArg);
      }
      else if ((currentState == ATLIST) && keyWord(word[i],"at"))
      { currentState = ATLIST;
        newTerm = register.getArgument(1);
        newArg = newTerm.getArgument(0);
        newArg.addWord(word[i]);
      }
     else if ((currentState == ATLIST) && quantifierLeast(word[i]))
      { currentState = INEQUALITYLIST;
        newTerm = register.getArgument(1);
        newTerm.setFunctor(OwlTerm.OBJECT_MIN_CARDINALITY);
        newArg = new OwlTerm(OwlTerm.CLASS,0);
        newTerm.setArgument(1,newArg);
      }
     else if ((currentState == ATLIST) && quantifierMost(word[i]))
      { currentState = INEQUALITYLIST;
        newTerm = register.getArgument(1);
        newTerm.setFunctor(OwlTerm.OBJECT_MAX_CARDINALITY);
        newArg = new OwlTerm(OwlTerm.CLASS,0);
        newTerm.setArgument(1,newArg);
      }
      else if ((currentState == ATLIST) && quantifierOnly(word[i]))
      { currentState = CLIST;
        newTerm = register.getArgument(1);
        newTerm.setFunctor(OwlTerm.OBJECT_ALL_VALUES_FROM);
        newArg = newTerm.getArgument(0);
        newArg.addWord(word[i-1]); // incorporate previous word "at"
        newArg = new OwlTerm(OwlTerm.CLASS,0);
        newTerm.setArgument(1,newArg);
        currentNumber = OwlEnglish.PLURAL;
      }
      else if ((currentState == ATLIST) && quantifierExactly(word[i]))
      { currentState = EQUALITYLIST;
        newTerm = register.getArgument(1);
        newTerm.setFunctor(OwlTerm.OBJECT_EXACT_CARDINALITY);
        newArg = newTerm.getArgument(0);
        newArg.addWord(word[i-1]); // incorporate previous word "at"
        newArg = new OwlTerm(OwlTerm.CLASS,0);
        newTerm.setArgument(1,newArg);
      }
      else if ((currentState == ATLIST) && indefiniteArticle(word[i]))
      { currentState = CLIST;
        newTerm = register.getArgument(1);
        newTerm.setFunctor(OwlTerm.OBJECT_SOME_VALUES_FROM);
        newArg = newTerm.getArgument(0);
        newArg.addWord(word[i-1]); // incorporate previous word "at"
        newArg = new OwlTerm(OwlTerm.CLASS,0);
        newTerm.setArgument(1,newArg);
        currentNumber = OwlEnglish.SINGULAR;
      }
     else if ((currentState == ATLIST) && verbPhraseContinuation(word[i]))
      { currentState = VPLIST;
        newTerm = register.getArgument(1);
        newArg = newTerm.getArgument(0);
        newArg.addWord(word[i-1]); // incorporate previous word "at"
        newArg.addWord(word[i]);
      }
      else if ((currentState == EQUALITYLIST) && integer(word[i]))
      { currentState = CLIST;
        newTerm = register.getArgument(1);
        newTerm.setEntityName(word[i].getWordString());
        if (singularInteger(word[i]) == true) currentNumber = OwlEnglish.SINGULAR;
        else currentNumber = OwlEnglish.PLURAL;
      }
      else if ((currentState == INEQUALITYLIST) && integer(word[i]))
      { currentState = CLIST;
        newTerm = register.getArgument(1);
        newTerm.setEntityName(word[i].getWordString());
        if (singularInteger(word[i]) == true) currentNumber = OwlEnglish.SINGULAR;
        else currentNumber = OwlEnglish.PLURAL;
      }
      else if ((currentState == ILIST) && nounPhraseContinuation(word[i]))
      { currentState = ILISTOBJ;
        newTerm = register.getArgument(1);
        newArg = newTerm.getArgument(1);
        newArg.addWord(word[i]);
      }
      else if ((currentState == CLIST) && nounPhraseContinuation(word[i]))
      { currentState = CLISTOBJ;
        newTerm = register.getArgument(1);
        newArg = newTerm.getArgument(1);
        newArg.addWord(word[i]);
      }
      else if ((currentState == VPLIST) && properName(word[i]))
      { currentState = ILISTOBJ;
        newTerm = register.getArgument(1);
        newTerm.setFunctor(OwlTerm.OBJECT_HAS_VALUE);
        newArg = new OwlTerm(OwlTerm.NAMED_INDIVIDUAL,0);
        newArg.addWord(word[i]);
        newTerm.setArgument(1,newArg);
      }
      else if ((currentState == VPLIST) && literal(word[i]))
      { currentState = LLISTOBJ;
        newTerm = register.getArgument(1);
        newTerm.setFunctor(OwlTerm.DATA_HAS_VALUE);
        newArg = new OwlTerm(OwlTerm.LITERAL,0);
        newArg.addWord(word[i]);
        newTerm.setArgument(1,newArg);
        newTerm.getArgument(0).setFunctor(OwlTerm.DATA_PROPERTY);
      }
      else if ((currentState == ILISTOBJ) && nounPhraseContinuation(word[i]))
      { currentState = ILISTOBJ;
        newTerm = register.getArgument(1);
        newArg = newTerm.getArgument(1);
        newArg.addWord(word[i]);
      }
      else if ((currentState == CLISTOBJ) && nounPhraseContinuation(word[i]))
      { currentState = CLISTOBJ;
        newTerm = register.getArgument(1);
        newArg = newTerm.getArgument(1);
        newArg.addWord(word[i]);
      }
      else if ((currentState == ILISTOBJ) && fullStop(word[i]))
      { currentState = FINISH;
      }
      else if ((currentState == LLISTOBJ) && fullStop(word[i]))
      { currentState = FINISH;
      }
      else if ((currentState == CLISTOBJ) && fullStop(word[i]))
      { currentState = FINISH;
      }
      else if ((currentState == ILISTOBJ) && conjunctionAnd(word[i]) && (listType != UNION))
      { currentState = VPLISTLOOP;
        listType = INTERSECTION;
        newTerm = new OwlTerm(OwlTerm.OBJECT_INTERSECTION_OF,2);
        newTerm.setArgument(0,register.getArgument(1));
        register.setArgument(1,newTerm);
        register = newTerm;
      }
      else if ((currentState == LLISTOBJ) && conjunctionAnd(word[i]) && (listType != UNION))
      { currentState = VPLISTLOOP;
        listType = INTERSECTION;
        newTerm = new OwlTerm(OwlTerm.OBJECT_INTERSECTION_OF,2);
        newTerm.setArgument(0,register.getArgument(1));
        register.setArgument(1,newTerm);
        register = newTerm;
      }
      else if ((currentState == CLISTOBJ) && conjunctionAnd(word[i]) && (listType != UNION))
      { currentState = VPLISTLOOP;
        listType = INTERSECTION;
        newTerm = new OwlTerm(OwlTerm.OBJECT_INTERSECTION_OF,2);
        newTerm.setArgument(0,register.getArgument(1));
        register.setArgument(1,newTerm);
        register = newTerm;
        currentNumber = OwlEnglish.SINGULAR;
      }
      else if ((currentState == ILISTOBJ) && conjunctionOr(word[i]) && (listType != INTERSECTION))
      { currentState = VPLISTLOOP;
        listType = UNION;
        newTerm = new OwlTerm(OwlTerm.OBJECT_UNION_OF,2);
        newTerm.setArgument(0,register.getArgument(1));
        register.setArgument(1,newTerm);
        register = newTerm;
      }
      else if ((currentState == LLISTOBJ) && conjunctionOr(word[i]) && (listType != INTERSECTION))
      { currentState = VPLISTLOOP;
        listType = UNION;
        newTerm = new OwlTerm(OwlTerm.OBJECT_UNION_OF,2);
        newTerm.setArgument(0,register.getArgument(1));
        register.setArgument(1,newTerm);
        register = newTerm;
      }
      else if ((currentState == CLISTOBJ) && conjunctionOr(word[i]) && (listType != INTERSECTION))
      { currentState = VPLISTLOOP;
        listType = UNION;
        newTerm = new OwlTerm(OwlTerm.OBJECT_UNION_OF,2);
        newTerm.setArgument(0,register.getArgument(1));
        register.setArgument(1,newTerm);
        register = newTerm;
        currentNumber = OwlEnglish.SINGULAR;
      }
      else if ((currentState == VPLISTLOOP) && verbPhraseOpening12(word[i]))
      { currentState = AUXLIST;
        newTerm = new OwlTerm(OwlTerm.UNSPECIFIED,2);
        newArg = new OwlTerm(OwlTerm.OBJECT_PROPERTY,0);
        newArg.addWord(word[i]);
        newTerm.setArgument(0,newArg);
        register.setArgument(1,newTerm);
      }
      else if ((currentState == VPLISTLOOP) && verbPhraseOpening3(word[i]))
      { currentState = VPLIST;
        newTerm = new OwlTerm(OwlTerm.UNSPECIFIED,2);
        newArg = new OwlTerm(OwlTerm.OBJECT_PROPERTY,0);
        newArg.addWord(word[i]);
        newTerm.setArgument(0,newArg);
        register.setArgument(1,newTerm);
      }
      else if ((currentState == AUXLIST) && verbPhraseContinuation(word[i]))
      { currentState = VPLIST;
        newTerm = register.getArgument(1);
        newArg = newTerm.getArgument(0);
        newArg.addWord(word[i]);
      }
      else if ((currentState == COMPLEMENT) && conjunctionThat(word[i]) && (listType != UNION))
      { currentState = VPLISTLOOP;
        listType = INTERSECTION;
        newTerm = new OwlTerm(OwlTerm.OBJECT_INTERSECTION_OF,2);
        newTerm.setArgument(0,register.getArgument(1));
        register.setArgument(1,newTerm);
        register = newTerm;
      }
      else if ((currentState == CLISTOBJ) && conjunctionThat(word[i]) && (listType != UNION))
      { currentState = VPCHAINLOOP;
        listType = INTERSECTION;
        newTerm = new OwlTerm(OwlTerm.OBJECT_INTERSECTION_OF,2);
        newTerm.setArgument(0,register.getArgument(1).getArgument(1));
        register.getArgument(1).setArgument(1,newTerm);
        register = newTerm;
      }
      else if ((currentState == VPCHAINLOOP) && verbPhraseOpening12(word[i]))
      { currentState = AUXCHAIN;
        newTerm = new OwlTerm(OwlTerm.UNSPECIFIED,2);
        newArg = new OwlTerm(OwlTerm.OBJECT_PROPERTY,0);
        newArg.addWord(word[i]);
        newTerm.setArgument(0,newArg);
        register.setArgument(1,newTerm);
      }
      else if ((currentState == VPCHAINLOOP) && verbPhraseOpening3(word[i]))
      { currentState = VPCHAIN;
        newTerm = new OwlTerm(OwlTerm.UNSPECIFIED,2);
        newArg = new OwlTerm(OwlTerm.OBJECT_PROPERTY,0);
        newArg.addWord(word[i]);
        newTerm.setArgument(0,newArg);
        register.setArgument(1,newTerm);
      }
      else if ((currentState == AUXCHAIN) && verbPhraseContinuation(word[i]))
      { currentState = VPCHAIN;
        newTerm = register.getArgument(1);
        newArg = newTerm.getArgument(0);
        newArg.addWord(word[i]);
      }
      else if ((currentState == VPCHAIN) && keyWord(word[i],"at"))
      { currentState = ATCHAIN;
      }
      else if ((currentState == VPCHAIN) && quantifierExactly(word[i]))
      { currentState = EQUALITYCHAIN;
        newTerm = register.getArgument(1);
        newTerm.setFunctor(OwlTerm.OBJECT_EXACT_CARDINALITY);
        newArg = new OwlTerm(OwlTerm.CLASS,0);
        newTerm.setArgument(1,newArg);
      }
      else if ((currentState == VPCHAIN) && quantifierOnly(word[i]))
      { currentState = CCHAIN;
        newTerm = register.getArgument(1);
        newTerm.setFunctor(OwlTerm.OBJECT_ALL_VALUES_FROM);
        newArg = new OwlTerm(OwlTerm.CLASS,0);
        newTerm.setArgument(1,newArg);
        currentNumber = OwlEnglish.PLURAL;
      }
      else if ((currentState == VPCHAIN) && verbPhraseContinuation(word[i]))
      { currentState = VPCHAIN;
        newTerm = register.getArgument(1);
        newArg = newTerm.getArgument(0);
        newArg.addWord(word[i]);
      }
      else if ((currentState == VPCHAIN) && definiteArticle(word[i]))
      { currentState = ICHAIN;
        newTerm = register.getArgument(1);
        newTerm.setFunctor(OwlTerm.OBJECT_HAS_VALUE);
        newArg = new OwlTerm(OwlTerm.NAMED_INDIVIDUAL,0);
        newArg.addWord(word[i]);
        newTerm.setArgument(1,newArg);
      }
      else if ((currentState == ATCHAIN) && keyWord(word[i],"at"))
      { currentState = ATCHAIN;
        newTerm = register.getArgument(1);
        newArg = newTerm.getArgument(0);
        newArg.addWord(word[i]);
      }
     else if ((currentState == ATCHAIN) && quantifierLeast(word[i]))
      { currentState = INEQUALITYCHAIN;
        newTerm = register.getArgument(1);
        newTerm.setFunctor(OwlTerm.OBJECT_MIN_CARDINALITY);
        newArg = new OwlTerm(OwlTerm.CLASS,0);
        newTerm.setArgument(1,newArg);
      }
     else if ((currentState == ATCHAIN) && quantifierMost(word[i]))
      { currentState = INEQUALITYCHAIN;
        newTerm = register.getArgument(1);
        newTerm.setFunctor(OwlTerm.OBJECT_MAX_CARDINALITY);
        newArg = new OwlTerm(OwlTerm.CLASS,0);
        newTerm.setArgument(1,newArg);
      }
      else if ((currentState == ATCHAIN) && quantifierOnly(word[i]))
      { currentState = CCHAIN;
        newTerm = register.getArgument(1);
        newTerm.setFunctor(OwlTerm.OBJECT_ALL_VALUES_FROM);
        newArg = newTerm.getArgument(0);
        newArg.addWord(word[i-1]);
        newArg = new OwlTerm(OwlTerm.CLASS,0);
        newTerm.setArgument(1,newArg);
        currentNumber = OwlEnglish.PLURAL;
      }
      else if ((currentState == ATCHAIN) && quantifierExactly(word[i]))
      { currentState = EQUALITYCHAIN;
        newTerm = register.getArgument(1);
        newTerm.setFunctor(OwlTerm.OBJECT_EXACT_CARDINALITY);
        newArg = newTerm.getArgument(0);
        newArg.addWord(word[i-1]);
        newArg = new OwlTerm(OwlTerm.CLASS,0);
        newTerm.setArgument(1,newArg);
      }
      else if ((currentState == ATCHAIN) && indefiniteArticle(word[i]))
      { currentState = CCHAIN;
        newTerm = register.getArgument(1);
        newTerm.setFunctor(OwlTerm.OBJECT_SOME_VALUES_FROM);
        newArg = newTerm.getArgument(0);
        newArg.addWord(word[i-1]);
        newArg = new OwlTerm(OwlTerm.CLASS,0);
        newTerm.setArgument(1,newArg);
        currentNumber = OwlEnglish.SINGULAR;
      }
     else if ((currentState == ATCHAIN) && verbPhraseContinuation(word[i]))
      { currentState = VPCHAIN;
        newTerm = register.getArgument(1);
        newArg = newTerm.getArgument(0);
        newArg.addWord(word[i-1]);
        newArg.addWord(word[i]);
      }
      else if ((currentState == EQUALITYCHAIN) && integer(word[i]))
      { currentState = CCHAIN;
        newTerm = register.getArgument(1);
        newTerm.setEntityName(word[i].getWordString());
        if (singularInteger(word[i]) == true) currentNumber = OwlEnglish.SINGULAR;
        else currentNumber = OwlEnglish.PLURAL;
      }
      else if ((currentState == INEQUALITYCHAIN) && integer(word[i]))
      { currentState = CCHAIN;
        newTerm = register.getArgument(1);
        newTerm.setEntityName(word[i].getWordString());
        if (singularInteger(word[i]) == true) currentNumber = OwlEnglish.SINGULAR;
        else currentNumber = OwlEnglish.PLURAL;
      }
      else if ((currentState == ICHAIN) && nounPhraseContinuation(word[i]))
      { currentState = ICHAINOBJ;
        newTerm = register.getArgument(1);
        newArg = newTerm.getArgument(1);
        newArg.addWord(word[i]);
      }
      else if ((currentState == VPCHAIN) && indefiniteArticle(word[i]))
      { currentState = CCHAIN;
        newTerm = register.getArgument(1);
        newTerm.setFunctor(OwlTerm.OBJECT_SOME_VALUES_FROM);
        newArg = new OwlTerm(OwlTerm.CLASS,0);
        newTerm.setArgument(1,newArg);
        currentNumber = OwlEnglish.SINGULAR;
      }
      else if ((currentState == CCHAIN) && nounPhraseContinuation(word[i]))
      { currentState = CCHAINOBJ;
        newTerm = register.getArgument(1);
        newArg = newTerm.getArgument(1);
        newArg.addWord(word[i]);
      }
      else if ((currentState == VPCHAIN) && properName(word[i]))
      { currentState = ICHAINOBJ;
        newTerm = register.getArgument(1);
        newTerm.setFunctor(OwlTerm.OBJECT_HAS_VALUE);
        newArg = new OwlTerm(OwlTerm.NAMED_INDIVIDUAL,0);
        newArg.addWord(word[i]);
        newTerm.setArgument(1,newArg);
      }
      else if ((currentState == VPCHAIN) && literal(word[i]))
      { currentState = LCHAINOBJ;
        newTerm = register.getArgument(1);
        newTerm.setFunctor(OwlTerm.DATA_HAS_VALUE);
        newArg = new OwlTerm(OwlTerm.LITERAL,0);
        newArg.addWord(word[i]);
        newTerm.setArgument(1,newArg);
        newTerm.getArgument(0).setFunctor(OwlTerm.DATA_PROPERTY);
      }
      else if ((currentState == ICHAINOBJ) && nounPhraseContinuation(word[i]))
      { currentState = ICHAINOBJ;
        newTerm = register.getArgument(1);
        newArg = newTerm.getArgument(1);
        newArg.addWord(word[i]);
      }
      else if ((currentState == CCHAINOBJ) && nounPhraseContinuation(word[i]))
      { currentState = CCHAINOBJ;
        newTerm = register.getArgument(1);
        newArg = newTerm.getArgument(1);
        newArg.addWord(word[i]);
      }
      else if ((currentState == ICHAINOBJ) && fullStop(word[i]))
      { currentState = FINISH;
      }
      else if ((currentState == LCHAINOBJ) && fullStop(word[i]))
      { currentState = FINISH;
      }
      else if ((currentState == CCHAINOBJ) && fullStop(word[i]))
      { currentState = FINISH;
      }
      else if ((currentState == CCHAINOBJ) && conjunctionThat(word[i]) & (listType != UNION))
      { currentState = VPCHAINLOOP;
        listType = INTERSECTION;
        newTerm = new OwlTerm(OwlTerm.OBJECT_INTERSECTION_OF,2);
        newTerm.setArgument(0,register.getArgument(1).getArgument(1));
        register.getArgument(1).setArgument(1,newTerm);
        register = newTerm;
      }
// Transitions for comments and metadata statements
      else if (currentState == COMMENT) 
      { currentState = COMMENT;
        if (i == numberOfWords-1) currentState = FINISH;
        namepart = register.getEntityName();
        if (namepart.equals("")) namepart = word[i].getWordString();
        else namepart = namepart + " " + word[i].getWordString();
        register.setEntityName(namepart);
      }
      else if (currentState == METADATA) 
      { currentState = METADATA;
        if (i == numberOfWords-1) currentState = FINISH;
        namepart = register.getEntityName();
        if (namepart.equals("")) namepart = word[i].getWordString();
        else namepart = namepart + " " + word[i].getWordString();
        register.setEntityName(namepart);
      }
      else currentState = REJECT;

// Assign entry and exit states, role and number to the word just transduced
      word[i].setWordEntryState(lastState);
      word[i].setWordExitState(currentState);
      word[i].setWordRole(getRole(lastState,currentState));
      word[i].setWordNumber(currentNumber);

// Reassign roles for current and previous word in five special cases
// First is when current role is same as previous
// Second is when "is" as previous word is revised from SUGAR to PROPERTY
// Third is when "at" as previous word is revised from PROPERTY to SUGAR
// Fourth, when "is defined" previously is revised from SUGAR to PROPERTY
// Fifth, when "The property" previously is revised from INDIVIDUAL to SUGAR

      if (word[i].getWordRole() == RETAIN)
      word[i].setWordRole(word[i-1].getWordRole());
      else if (word[i].getWordRole() == REVISE)
      { word[i].setWordRole(PROPERTY);
        word[i-1].setWordRole(PROPERTY);
      }
      else if (word[i].getWordRole() == SUGARBACK)
      { word[i].setWordRole(SUGAR);
        word[i-1].setWordRole(SUGAR);
      }
      else if (word[i].getWordRole() == REVISEBACK)
      { word[i].setWordRole(PROPERTY);
        word[i-1].setWordRole(PROPERTY);
        word[i-2].setWordRole(PROPERTY);
      }
      else if (word[i].getWordRole() == SUGARBACKTWO)
      { word[i].setWordRole(SUGAR);
        word[i-1].setWordRole(SUGAR);
        word[i-2].setWordRole(SUGAR);
      }

// Note whether sentence is correctly completed
      if (currentState == FINISH) correctSentence = true;
    }
  }

  private int attributeFunctor(Word word)
  { int functor = 0;
    String s = word.getWordString();
    if (s.equals("functional"))
      functor = OwlTerm.FUNCTIONAL_OBJECT_PROPERTY;
    else if (s.equals("inverse-functional"))
      functor = OwlTerm.INVERSE_FUNCTIONAL_OBJECT_PROPERTY;
    else if (s.equals("symmetric"))
      functor = OwlTerm.SYMMETRIC_OBJECT_PROPERTY;
    else if (s.equals("asymmetric"))
      functor = OwlTerm.ASYMMETRIC_OBJECT_PROPERTY;
    else if (s.equals("reflexive"))
      functor = OwlTerm.REFLEXIVE_OBJECT_PROPERTY;
    else if (s.equals("irreflexive"))
      functor = OwlTerm.IRREFLEXIVE_OBJECT_PROPERTY;
    else if (s.equals("transitive"))
      functor = OwlTerm.TRANSITIVE_OBJECT_PROPERTY;
    return functor;
  }

  private boolean propertyAttribute(Word word)
  { boolean tval = false;
    String s = word.getWordString();
    if (s.equals("functional") ||
        s.equals("inverse-functional") ||
        s.equals("symmetric") ||
        s.equals("asymmetric") ||
        s.equals("reflexive") ||
        s.equals("irreflexive") ||
        s.equals("transitive") ||
        s.equals("[attribute]"))
    tval = true;
    return tval;
  }

  private int relationFunctor(Word word)
  { int functor = 0;
    String s = word.getWordString();
    if (s.equals("disjoint"))
      functor = OwlTerm.DISJOINT_OBJECT_PROPERTIES;
    else if (s.equals("equivalent"))
      functor = OwlTerm.EQUIVALENT_OBJECT_PROPERTIES;
    else if (s.equals("inverse") || s.equals("inverses"))
      functor = OwlTerm.INVERSE_OBJECT_PROPERTIES;
    return functor;
  }

  private boolean propertyRelation(Word word)
  { boolean tval = false;
    String s = word.getWordString();
    if (s.equals("disjoint") ||
        s.equals("equivalent") ||
        s.equals("inverse") ||
        s.equals("inverses"))
    tval = true;
    return tval;
  }

  private int dataRelationFunctor(Word word)
  { int functor = 0;
    String s = word.getWordString();
    if (s.equals("disjoint"))
      functor = OwlTerm.DISJOINT_DATA_PROPERTIES;
    else if (s.equals("equivalent"))
      functor = OwlTerm.EQUIVALENT_DATA_PROPERTIES;
    return functor;
  }

  private boolean dataPropertyRelation(Word word)
  { boolean tval = false;
    String s = word.getWordString();
    if (s.equals("disjoint") || s.equals("equivalent"))
    tval = true;
    return tval;
  }

  private boolean singularInteger(Word word)
  { boolean tval = false;
    String s = word.getWordString();
    if (s.equals("one") || s.equals("1")) tval = true;
    return tval;
  }

  private boolean keyWord(Word word, String key)
  { boolean tval = false;
    if (word.getWordString().equals(key)) tval = true;  
    return tval; 
  }

  private boolean definiteArticle(Word word)
  { boolean tval = false;
    if (word.getWordCategory() == Word.DEFINITE_ARTICLE) tval = true;  
    return tval; 
  }

  private boolean indefiniteArticle(Word word)
  { boolean tval = false;
    if (word.getWordCategory() == Word.INDEFINITE_ARTICLE) tval = true;  
    return tval; 
  }

  private boolean determinerEvery(Word word)
  { boolean tval = false;
    if (word.getWordCategory() == Word.INDEFINITE_ARTICLE) tval = true;
    if (word.getWordCategory() == Word.DETERMINER_EVERY) tval = true; 
    return tval; 
  }

  private boolean determinerNo(Word word)
  { boolean tval = false;
    if (word.getWordCategory() == Word.DETERMINER_NO) tval = true; 
    return tval; 
  }

  private boolean determinerAny(Word word)
  { boolean tval = false;
    if (word.getWordCategory() == Word.DETERMINER_ANY) tval = true; 
    return tval; 
  }

  private boolean quantifierAnything(Word word)
  { boolean tval = false;
    if (word.getWordCategory() == Word.QUANTIFIER_ANYTHING) tval = true; 
    return tval; 
  }

  private boolean quantifierSomething(Word word)
  { boolean tval = false;
    if (word.getWordCategory() == Word.QUANTIFIER_SOMETHING) tval = true; 
    return tval; 
  }

  private boolean quantifierSome(Word word)
  { boolean tval = false;
    if (word.getWordCategory() == Word.QUANTIFIER_SOME) tval = true; 
    return tval; 
  }

  private boolean quantifierOnly(Word word)
  { boolean tval = false;
    if (word.getWordCategory() == Word.QUANTIFIER_ONLY) tval = true; 
    return tval; 
  }

  private boolean quantifierMost(Word word)
  { boolean tval = false;
    if (word.getWordCategory() == Word.QUANTIFIER_MOST) tval = true; 
    return tval; 
  }

  private boolean quantifierLeast(Word word)
  { boolean tval = false;
    if (word.getWordCategory() == Word.QUANTIFIER_LEAST) tval = true; 
    return tval; 
  }

  private boolean quantifierExactly(Word word)
  { boolean tval = false;
    if (word.getWordCategory() == Word.QUANTIFIER_EXACTLY) tval = true; 
    return tval; 
  }

  private boolean properName(Word word)
  { boolean tval = false;
    if (word.getWordCategory() == Word.PROPER_NAME) tval = true;  
    return tval;
  }

  private boolean literal(Word word)
  { boolean tval = false;
    if (word.getWordCategory() == Word.STRING) tval = true;  
    if (word.getWordCategory() == Word.INTEGER) tval = true;  
    if (word.getWordCategory() == Word.DECIMAL) tval = true;  
    return tval;
  }

  private boolean integer(Word word)
  { boolean tval = false;
    if (word.getWordCategory() == Word.INTEGER) tval = true;  
    return tval;
  }

  private boolean conjunctionAnd(Word word)
  { boolean tval = false;
    if (word.getWordString().equals("and")) tval = true;  
    return tval; 
  }

  private boolean conjunctionOr(Word word)
  { boolean tval = false;
    if (word.getWordString().equals("or")) tval = true;  
    return tval; 
  }

  private boolean conjunctionThat(Word word)
  { boolean tval = false;
    if (word.getWordString().equals("that")) tval = true;  
    return tval; 
  }

  private boolean openAngleBracket(Word word)
  { boolean tval = false;
    if (word.getWordCategory() == Word.OPEN_ANGLE_BRACKET) tval = true;  
    return tval; 
  }

  private boolean closeAngleBracket(Word word)
  { boolean tval = false;
    if (word.getWordCategory() == Word.CLOSE_ANGLE_BRACKET) tval = true;  
    return tval; 
  }

  private boolean nounPhraseContinuation(Word word)
  { boolean tval = false;
    if (definiteArticle(word) == true) tval = true;
    if (word.getWordCategory() == Word.NOUN) tval = true;  
    if (word.getWordCategory() == Word.PROPER_NAME) tval = true;  
    if (word.getWordCategory() == Word.STRING) tval = true;  
    if (word.getWordCategory() == Word.INTEGER) tval = true;  
    if (word.getWordCategory() == Word.DECIMAL) tval = true;  
    if (word.getWordCategory() == Word.PREPOSITION) tval = true;
    return tval;
  }

 private boolean verbPhraseOpening1(Word word)
  { boolean tval = false;
    if (word.getWordCategory() == Word.AUXILIARY_IS) tval = true;
    return tval; 
  }

 private boolean verbPhraseOpening2(Word word)
  { boolean tval = false;
    if (word.getWordCategory() == Word.AUXILIARY_HAS) tval = true;
    return tval; 
  }

 private boolean verbPhraseOpening3(Word word)
  { boolean tval = false;
    if (word.getWordCategory() == Word.VERB) tval = true;
    return tval; 
  }

  private boolean verbPhraseOpening12(Word word)
  { boolean tval = false;
    if (verbPhraseOpening1(word) || verbPhraseOpening2(word)) tval = true;
    return tval; 
  }

  private boolean verbPhraseContinuation(Word word)
  { boolean tval = false;
    if (word.getWordCategory() == Word.NOUN) tval = true;  
    if (word.getWordCategory() == Word.PREPOSITION) tval = true;
    return tval;
  }

  private boolean fullStop(Word word)
  { boolean tval = false;
    if (word.getWordCategory() == Word.FULL_STOP) tval = true;  
    return tval;
  }

  private boolean isCommentOrMetadata(OwlTerm term)
  { boolean tval = false;
    int functor = term.getFunctor();
    if ((functor == OwlTerm.COMMENT)  || (functor == OwlTerm.METADATA))
    tval = true;
    return tval;
  }

  public void display()
  { System.out.println("\nSentence: " + sentenceNumber);
    System.out.println("Text: " + textString);
    System.out.println("Start: " + startPosition);
    System.out.println("End: " + endPosition);
    System.out.println("Number of characters: " + numberOfCharacters);
    for (int i=0; i<word.length; i++) word[i].display();
    System.out.println("Number of words: " + numberOfWords);
  }
}
