
/*
Instances of Word hold information about each word in a sentence

Word objects comprise a string holding the spelling of the word, a
syntactic category (using a simplified scheme in which NOUN also
covers adjectives, adverbs, past participles), and the states of the
finite-state transducer that are traversed before and after the word
is consumed.

The states and role are assigned by a Sentence method which applies
the FS transducer to the whole sentence. The number (SINGULAR/PLURAL)
is also set by the transducer to distinguish singular from plural
contexts. This guides the choice of option lists for a place-holder.

*/

import java.util.Vector;
import java.util.Collections;
import java.lang.String;

public class Word
{ private String wordString;
  private int numberOfCharacters;
  private int wordCategory, wordEntryState, wordExitState, wordRole, wordNumber;
  private int sentencePosition, wordStart, wordFinish;
  private Vector<Verb> verbs;
  private String wordSingular;
  private String wordPlural;

// Categories for words (parts of speech)
  public static final int DEFINITE_ARTICLE = 0;
  public static final int INDEFINITE_ARTICLE = 1;
  public static final int AUXILIARY_IS = 2;
  public static final int CONJUNCTION = 3;
  public static final int PREPOSITION = 4;
  public static final int VERB = 5;
  public static final int NOUN = 6;
  public static final int PROPER_NAME = 7;
  public static final int STRING = 8;
  public static final int INTEGER = 9;
  public static final int DECIMAL = 10;
  public static final int FULL_STOP = 11;
  public static final int DETERMINER_EVERY = 12;
  public static final int DETERMINER_NO = 13;
  public static final int DETERMINER_ANY = 14;
  public static final int QUANTIFIER_ANYTHING = 15;
  public static final int QUANTIFIER_SOMETHING = 16;
  public static final int QUANTIFIER_SOME = 17;
  public static final int QUANTIFIER_ONLY = 18;
  public static final int QUANTIFIER_EXACTLY = 19;
  public static final int QUANTIFIER_MOST = 20;
  public static final int QUANTIFIER_LEAST = 21;
  public static final int AUXILIARY_HAS = 22;
  public static final int OPEN_ANGLE_BRACKET = 23;
  public static final int CLOSE_ANGLE_BRACKET = 24;

// Print labels for categories
  private static String pos[] = { "definite article",
                                  "indefinite article",
                                  "auxiliary IS",
                                  "conjunction",
                                  "preposition",
                                  "verb",
                                  "noun",
                                  "proper name",
                                  "string",
                                  "integer",
                                  "decimal",
                                  "full stop",
                                  "determiner EVERY",
                                  "determiner NO",
                                  "determiner ANY",
                                  "quantifier ANYTHING",
                                  "quantifier SOMETHING",
                                  "quantifier SOME",
                                  "quantifier ONLY",
                                  "quantifier EXACTLY",
                                  "quantifier MOST",
                                  "quantifier LEAST",
                                  "auxiliary HAS",
                                  "open angle bracket",
                                  "close angle bracket" };

// List of prepositions
  private static String preposition[] =
    { "about",
      "above",
      "across",
      "afore",
      "after",
      "against",
      "along",
      "alongside",
      "amid",
      "among",
      "amongst",
      "apropos",
      "around",
      "as",
      "aside",
      "astride",
      "at",
      "athwart",
      "before",
      "behind",
      "below",
      "beneath",
      "beside",
      "between",
      "beyond",
      "by",
      "circa",
      "down",
      "during",
      "for",
      "from",
      "in",
      "into",
      "of",
      "on",
      "onto",
      "over",
      "per",
      "to",
      "under",
      "up",
      "with",
      "within"
    };

// List of integer words
  public static String integerWord[] =
    { "zero",
      "one",
      "two",
      "three",
      "four",
      "five",
      "six",
      "seven",
      "eight",
      "nine",
      "ten"
    };

// List of property attribute words
  public static String attributeWord[] =
    { "asymmetric",
      "functional",
      "inverse-functional",
      "irreflexive",
      "reflexive",
      "symmetric",
      "transitive"
    };

// Test cases
  private static String test[] =
    { ".",
      "into",
      "no",
      "any",
      "Every",
      "Anything",
      "something",
      "The",
      "has",
      "likes",
      "bus",
      "and",
      "Elizabeth",
      "\"Pride and Prejudice\"",
      "365",
      "3.14",
      "only",
      "exactly",
      "most",
      "least",
      "seven",
      "dishes",
      "women",
      "are",
      "have",
      "some",
      "<",
      ">"
    };

// Main method for testing
  public static void main(String[] args)
  { Word word;
    Vector<Verb> verbs = new Vector<Verb>();
    for (int i=0; i<test.length; i++)
    { word = new Word(i,test[i],2,3,verbs);
      word.display();
    }
  }

  public Word(int arg1, String arg2, int arg3, int arg4, Vector<Verb> arg5)
  { sentencePosition = arg1;
    wordString = arg2;
    wordStart = arg3;
    wordFinish = arg4;
    verbs = arg5;
    numberOfCharacters = wordString.length();
    wordCategory = assignWordCategory(wordString);
    findWordSingular();
    findWordPlural();
  }

  public void setWordString(String stringArg) {wordString = stringArg; }
  public void setWordRole(int intArg) {wordRole = intArg; }
  public void setWordNumber(int intArg) {wordNumber = intArg; }
  public void setWordEntryState(int intArg) {wordEntryState = intArg; }
  public void setWordExitState(int intArg) {wordExitState = intArg; }

  public String getWordString() {return wordString; }
  public int getWordCategory() {return wordCategory; }
  public int getWordEntryState() {return wordEntryState; }
  public int getWordExitState() {return wordExitState; }
  public int getWordStart() {return wordStart; }
  public int getWordFinish() {return wordFinish; }
  public int getWordRole() {return wordRole; }
  public int getWordNumber() {return wordNumber; }
  public int getSentencePosition() {return sentencePosition; }
  public String getWordSingular() {return wordSingular; }
  public String getWordPlural() {return wordPlural; }

// Uses large list of verbs in VerbList class
  private int assignWordCategory(String s)
  { int category = NOUN;
    if (userDefinedVerb(s) == true) category = VERB;
    if ((s.length() > 0) && Character.isUpperCase(s.charAt(0))) category = PROPER_NAME;
    if (s.equals(".")) category = FULL_STOP;
    if (s.equals("<")) category = OPEN_ANGLE_BRACKET;
    if (s.equals(">")) category = CLOSE_ANGLE_BRACKET;
    if (s.equals("A") || s.equals("An")) category = INDEFINITE_ARTICLE;
    if (s.equals("a") || s.equals("an")) category = INDEFINITE_ARTICLE;
    if (s.equals("No") || s.equals("no")) category = DETERMINER_NO;
    if (s.equals("Any") || s.equals("any")) category = DETERMINER_ANY;
    if (s.equals("Every") || s.equals("every")) category = DETERMINER_EVERY;
    if (s.equals("Anything") || s.equals("anything")) category = QUANTIFIER_ANYTHING;
    if (s.equals("Something") || s.equals("something")) category = QUANTIFIER_SOMETHING;
    if (s.equals("Some") || s.equals("some")) category = QUANTIFIER_SOME;
    if (s.equals("only")) category = QUANTIFIER_ONLY;
    if (s.equals("exactly")) category = QUANTIFIER_EXACTLY;
    if (s.equals("most")) category = QUANTIFIER_MOST;
    if (s.equals("least")) category = QUANTIFIER_LEAST;
    if (s.equals("the") || s.equals("The")) category = DEFINITE_ARTICLE;
    if (s.equals("is") || s.equals("are")) category = AUXILIARY_IS;
    if (s.equals("has") || s.equals("have")) category = AUXILIARY_HAS;
    if (s.equals("and") || s.equals("or") || s.equals("that")) category = CONJUNCTION;
    if (isPreposition(s)) category = PREPOSITION;
    if (s.startsWith("\"")) category = STRING;
    if (integerString(s) || isIntegerWord(s)) category = INTEGER;
    else if (decimalString(s)) category = DECIMAL;
    // Place holders
    if (s.equals("[Individual]")) category = PROPER_NAME;
    if (s.equals("[class]")) category = NOUN;
    if (s.equals("[has-property]")) category = VERB;
    if (s.equals("[has-data-property]")) category = VERB;
    if (s.equals("[literal]")) category = STRING;
    if (s.equals("[integer]")) category = INTEGER;
    return category;
  }

  private boolean isPreposition(String s)
  { boolean tval = false;
    for (int i=0; i<preposition.length; i++)
    if (preposition[i].equals(s)) tval = true;
    return tval;
  }

  private boolean isIntegerWord(String s)
  { boolean tval = false;
    for (int i=0; i<integerWord.length; i++)
    if (integerWord[i].equals(s)) tval = true;
    return tval;
  }

  private boolean integerString(String s)
  { boolean tval = true;
    int length = s.length();
    if (length > 0)
    { for (int i=0; i<length; i++)
      if (!Character.isDigit(s.charAt(i))) tval = false;
    }
    else tval = false;
    return tval;
  }

  private boolean decimalString(String s)
  { boolean tval = false;
    int length = s.length();
    if (length > 2)
    { for (int i=1; i<length; i++)
      if ((s.charAt(i) == '.') &&
          integerString(s.substring(0,i)) &&
          integerString(s.substring(i+1,length)))
      tval = true;
    }
    else tval = false;
    return tval;
  }

  private boolean userDefinedVerb(String s)
  { boolean tval = false;
    for (int i=0; i<verbs.size(); i++)
    { Verb verb = verbs.elementAt(i);
      if (verb.getSingular().equals(s) || verb.getPlural().equals(s)) tval = true;
    }
    return tval;
  }

// Find the singular form of a plural noun or verb
// The WordForm object applies a rule to derive singular from plural.
// This rule leaves the word unchanged if it is already singular.
  private void findWordSingular()
  { wordSingular = wordString;
    if (wordCategory == NOUN)
    { WordForm wordForm = new WordForm(wordString,WordForm.PLURAL);
      wordSingular = wordForm.getSingular();
    }
    else if (wordCategory == AUXILIARY_IS) wordSingular = "is";
    else if (wordCategory == AUXILIARY_HAS) wordSingular = "has";
    else if (wordCategory == VERB) wordSingular = singularVerbForm(wordString);
  }

// Find the plural form of a noun or verb
// Assumes singular form already computed
  private void findWordPlural()
  { wordPlural = wordSingular;
    if (wordCategory == NOUN)
    { WordForm wordForm = new WordForm(wordSingular,WordForm.SINGULAR);
      wordPlural = wordForm.getPlural();
    }
    else if (wordCategory == AUXILIARY_IS) wordPlural = "are";
    else if (wordCategory == AUXILIARY_HAS) wordPlural = "have";
    else if (wordCategory == VERB) wordPlural = pluralVerbForm(wordSingular);
  }

  private String singularVerbForm(String verb)
  { String stem = verb;
    for (int i=0; i<verbs.size(); i++)
    { Verb v = verbs.elementAt(i);
      if (v.getPlural().equals(verb)) stem = v.getSingular();
    }
    return stem;
  }

  private String pluralVerbForm(String singular)
  { String plural = singular;
    for (int i=0; i<verbs.size(); i++)
    { Verb v = verbs.elementAt(i);
      if (v.getSingular().equals(singular)) plural = v.getPlural();
    }
    return plural;
  }
    
  public void display()
  { System.out.println(sentencePosition + " " + 
                       wordString + " " + wordSingular + " " +
                       numberOfCharacters + " " + wordStart + "-" + 
                       wordFinish + " " + pos[wordCategory] + " " +
                       wordEntryState + "-" + wordExitState + " " + wordRole);
  }
    
}
